//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "AddThreadRange.h"
#include "GraphicsHelp.h"
#include "StringHelp.h"
#include <shellapi.h>
#include <IniFiles.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TAddThreadRangeForm *AddThreadRangeForm;
//---------------------------------------------------------------------------
__fastcall TAddThreadRangeForm::TAddThreadRangeForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TAddThreadRangeForm::FormCreate(TObject *Sender)
{
    //initialise some variables
    Application->OnHint = DisplayHint;
    shorttitlefound = false;

}
//---------------------------------------------------------------------------
void __fastcall TAddThreadRangeForm::FormDestroy(TObject *Sender)
{
    //empty arrays
    DMCColourList.Length = 0;
    for (int i = 0; i < TweedList.Length; i++) {
        TweedList[i].ColourPairs.Length = 0;
        }
    TweedList.Length = 0;
    Application->OnHint = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::ReadDMCColours(void)
{
    //This method opens the DMC colour list, and sets up the list as an array
    //The file is in the same directory as the application, and is a CSV file
    //AnsiString FileName = ExtractFileDir(Application->ExeName) + "\\dmc2a.csv";
    TStringList *inputfile = new TStringList();
    AnsiString ThreadFileName = FileName;
    //if (FileExists(FileName)) {
        //inputfile->LoadFromFile(FileName);
    if (FileExists(ThreadFileName)) {
        inputfile->LoadFromFile(ThreadFileName);
        //interpret file
        //first delete all comment lines
        for (int i = 0; i < inputfile->Count; i++) {
            if (LeftString(inputfile->Strings[i],1) == "#") {
                if (inputfile->Strings[i].Pos("Title=")){
                    ShortThreadName->Text = DeleteWord(inputfile->Strings[i],"=");
                    shorttitlefound = true;
                    ShortThreadName->Enabled = false;
                    ShortThreadName->Color = clBtnFace;
                    }
                inputfile->Delete(i);
                i--;
                }
            }
        //now create colour array & fill it
        DMCColourList.Length = inputfile->Count;
        AnsiString text;
        int value;
        rgb RGB;
        hsb HSB;
        for (int i = 0; i < inputfile->Count; i++) {
            text = inputfile->Strings[i];
            text = text + ",";
            //Extract the DMC Colour number. This is a string as 'Ecru' has no numeric meaning!
            DMCColourList[i].DMCColourNumber = ExtractWord(text,",");
            text = DeleteWord(text,",");
            //Extract the DMC Colour Name
            DMCColourList[i].DMCColourName = ExtractWord(text,",");
            text = DeleteWord(text,",");
            DMCColourList[i].DrawWhite = true;
            //Now to decide if the individual colour components exceed 100, if so, draw the
            //glyph in black on the background, otherwise use white
            //The half-way value is 128, but printers tend to print colours a bit light
            //so can go below 128 and still read the black symbol
            value = ExtractWord(text,",").ToIntDef(0);
            DMCColourList[i].R = value;
            if (value > 100) DMCColourList[i].DrawWhite = false;
            text = DeleteWord(text,",");
            value = ExtractWord(text,",").ToIntDef(0);
            DMCColourList[i].G = value;
            if (value > 100) DMCColourList[i].DrawWhite = false;
            text = DeleteWord(text,",");
            value = ExtractWord(text,",").ToIntDef(0);
            DMCColourList[i].B = value;
            if (value > 100) DMCColourList[i].DrawWhite = false;
            text = DeleteWord(text,",");
            DMCColourList[i].Colour = (TColor) ExtractWord(text,",").ToIntDef(0);
            text = DeleteWord(text,",");
            DMCColourList[i].StitchCount = 0;
            DMCColourList[i].AnchorColourNumber = ExtractWord(text,",");
            DMCColourList[i].AnchorColourToUse = 0;
            DMCColourList[i].AnchorStitchCount = 0;
            RGB.R = DMCColourList[i].R;
            RGB.G = DMCColourList[i].G;
            RGB.B = DMCColourList[i].B;
            HSB = RGBtoHSB(RGB);
            DMCColourList[i].Hue = HSB.Hue;
            DMCColourList[i].Saturation = HSB.Saturation;
            DMCColourList[i].Brightness = HSB.Brightness;
            }
        //finish interpretation
        inputfile->Clear();
        } else {
        AnsiString text = "Thread List (" + FileName + ") not found.";
        Application->MessageBox(text.c_str(),"Error",MB_ICONERROR|IDOK);
        }
    delete inputfile;

}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Initialise(void)
{
    Label5->Caption = FileName;
    Label6->Caption = ChangeFileExt(FileName,"Blends.csv");
    ThreadName->Text = ChangeFileExt(ExtractFileName(FileName),"");
    ShortThreadName->Text = LeftString(ThreadName->Text,6);
    ReadDMCColours();
    FillImage2();
    CalculateButtonClick(this);
    
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::DisplayHint(TObject *Sender)
{
    StatusBar1->SimpleText = " " + GetLongHint(Application->Hint);

}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::FillImage2(void)
{
    //set up DMC palette.
    Image2->Picture->Bitmap->Width = 150;
    Image2->Picture->Bitmap->Height = 10 * (DMCColourList.Length / 15 + 1);
    int pointer = 0;
    int j = 0;
    Image2->Picture->Bitmap->Canvas->Brush->Style = bsSolid;
    Image2->Picture->Bitmap->Canvas->Pen->Style = psClear;

    while (pointer < DMCColourList.Length) {
        for (int i = 0; i < 15; i++) {
            if (pointer < DMCColourList.Length) {
                Image2->Picture->Bitmap->Canvas->Brush->Color = DMCColourList[pointer].Colour;
                Image2->Picture->Bitmap->Canvas->Rectangle(i*10,j*10,((i+1)*10) - 1,((j+1)*10) - 1);
                }
            pointer++;
            }
        j++;
        }
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::CalculateButtonClick(TObject *Sender)
{
    //first calculates the blends, then fills in Image3
    Calculate();
    Label11->Caption = "Number of Blends = " + AnsiString(TweedList.Length);
    FillImage3();
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::FillImage3(void)
{
    //set up Blend palette.
    int pointer = 0;
    int j = 0;
    Image3->Picture->Bitmap->Width = 150;
    Image3->Picture->Bitmap->Height = 2 * (TweedList.Length / 75 + 1);
    pointer = 0;
    j = 0;
    Image3->Picture->Bitmap->Canvas->Brush->Style = bsSolid;
    Image3->Picture->Bitmap->Canvas->Pen->Style = psSolid;
    TColor Pixelcolour;

    while (pointer < TweedList.Length) {
        for (int i = 0; i < 75; i++) {
            if (pointer < TweedList.Length) {
                Pixelcolour = TweedList[pointer].Data.Colour;
                Image3->Picture->Bitmap->Canvas->Pixels[i*2][j*2] = Pixelcolour;
                Image3->Picture->Bitmap->Canvas->Pixels[i*2+1][j*2] = Pixelcolour;
                Image3->Picture->Bitmap->Canvas->Pixels[i*2][j*2+1] = Pixelcolour;
                Image3->Picture->Bitmap->Canvas->Pixels[i*2+1][j*2+1] = Pixelcolour;
                }
            pointer++;
            }
        j++;
        }
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Calculate(void)
{
    //creates a tweed list
    Screen->Cursor = crHourGlass;

    double maxangle = ToDoubleDef(Edit1->Text,90.0);
    double maxsatur = ToDoubleDef(Edit2->Text,0.5);
    double maxbrigh = ToDoubleDef(Edit3->Text,0.5);
    TweedList.Length = 100000;
    for (int i = 0; i < TweedList.Length; i++) {
        TweedList[i].ColourPairs.Length = 0;
        }
    int pointer = 0;

    double maxhue, minhue, swap, maxsat, minsat, maxbri, minbri;
    rgb RGB,RGB1,RGB2;
    hsb HSB;
    DMCColourData Tweed;
    bool addtolist;

    for (int i = 0; i < DMCColourList.Length - 1; i++) {
        for (int j = i + 1; j < DMCColourList.Length; j++) {
            maxhue = max(DMCColourList[i].Hue,DMCColourList[j].Hue);
            minhue = min(DMCColourList[i].Hue,DMCColourList[j].Hue);
            maxsat = max(DMCColourList[i].Saturation,DMCColourList[j].Saturation);
            minsat = min(DMCColourList[i].Saturation,DMCColourList[j].Saturation);
            maxbri = max(DMCColourList[i].Brightness,DMCColourList[j].Brightness);
            minbri = min(DMCColourList[i].Brightness,DMCColourList[j].Brightness);
            if ((maxhue - minhue) > 180.0) {
                swap = maxhue - 360;
                maxhue = minhue;
                minhue = swap;
                }
            if ((maxhue - minhue) < maxangle && (maxsat - minsat) < maxsatur && (maxbri - minbri) < maxbrigh) {
                //this tweed is a possible.
                Tweed.DMCColourNumber = DMCColourList[i].DMCColourNumber + "+" + DMCColourList[j].DMCColourNumber;
                Tweed.DMCColourName = "Blend of " + DMCColourList[i].DMCColourNumber + " + " + DMCColourList[j].DMCColourNumber;
                Tweed.AnchorColourNumber = "No Equivalent";
                RGB1 = ExtractRGB(DMCColourList[i].Colour);
                RGB2 = ExtractRGB(DMCColourList[j].Colour);
                RGB = MeanRGB(RGB1,RGB2);
                Tweed.Colour = GetColourFromRGB(RGB);
                if (RGB.R > 100 || RGB.G > 100 || RGB.B > 100) {
                    Tweed.DrawWhite = false;
                    } else {
                    Tweed.DrawWhite = true;
                    }
                Tweed.StitchCount = 0;
                Tweed.AnchorColourToUse = 0;
                Tweed.AnchorStitchCount = 0;
                Tweed.AnchorGlyphToUse = 0;
                Tweed.R = RGB.R;
                Tweed.G = RGB.G;
                Tweed.B = RGB.B;
                HSB = RGBtoHSB(RGB);
                Tweed.Hue = HSB.Hue;
                Tweed.Saturation = HSB.Saturation;
                Tweed.Brightness = HSB.Brightness;
                addtolist = true;
                //test if this tweed exists as a dmc colour
                int k = 0;
                while (addtolist && k < DMCColourList.Length) {
                    if (abs(Tweed.R - DMCColourList[k].R) < 2 && abs(Tweed.G - DMCColourList[k].G) < 2 && abs(Tweed.B - DMCColourList[k].B) < 2) {
                        addtolist = false;
                        }
                    k++;
                    }
                //test if it's in the tweed list
                k = 0;
                while (addtolist && k < pointer) {
                    if (abs(Tweed.R - TweedList[k].Data.R) < 2 && abs(Tweed.G - TweedList[k].Data.G) < 2 && abs(Tweed.B - TweedList[k].Data.B) < 2) {
                        addtolist = false;
                        TweedList[k].Data.DMCColourName = TweedList[k].Data.DMCColourName + " and " + DeleteWord(Tweed.DMCColourName,"Blend of ");
                        TweedList[k].ColourPairs.Length++;
                        TweedList[k].ColourPairs[TweedList[k].ColourPairs.Length - 1].x = i;
                        TweedList[k].ColourPairs[TweedList[k].ColourPairs.Length - 1].y = j;
                        }
                    k++;
                    }
                //if not, add
                if (addtolist) {
                    TweedList[pointer].Data = Tweed;
                    TweedList[pointer].ColourPairs.Length = 1;
                    TweedList[pointer].ColourPairs[0].x = i;
                    TweedList[pointer].ColourPairs[0].y = j;
                    pointer++;
                    if (pointer > TweedList.Length - 1) TweedList.Length += 10000;
                    if (!(pointer%1000)) {
                        Label11->Caption = "Number of Blends = " + AnsiString(pointer);
                        Refresh();
                        }
                    }
                }
            }
        }

    TweedList.Length = pointer;

    //now to sort into minimum colour differences
    int leastsquares1, leastsquares2;
    for (int i = 0; i < TweedList.Length; i++) {
        TPoint swap;
        bool swapped = true;
        while (swapped) {
            swapped = false;
            for (int j = 0; j < TweedList[i].ColourPairs.Length - 1; j++) {
                leastsquares1 = (DMCColourList[TweedList[i].ColourPairs[j].x].R - DMCColourList[TweedList[i].ColourPairs[j].y].R)
                                * (DMCColourList[TweedList[i].ColourPairs[j].x].R - DMCColourList[TweedList[i].ColourPairs[j].y].R)
                                + (DMCColourList[TweedList[i].ColourPairs[j].x].G - DMCColourList[TweedList[i].ColourPairs[j].y].G)
                                * (DMCColourList[TweedList[i].ColourPairs[j].x].G - DMCColourList[TweedList[i].ColourPairs[j].y].G)
                                + (DMCColourList[TweedList[i].ColourPairs[j].x].B - DMCColourList[TweedList[i].ColourPairs[j].y].B)
                                * (DMCColourList[TweedList[i].ColourPairs[j].x].B - DMCColourList[TweedList[i].ColourPairs[j].y].B)
                                ;
                leastsquares2 = (DMCColourList[TweedList[i].ColourPairs[j+1].x].R - DMCColourList[TweedList[i].ColourPairs[j+1].y].R)
                                * (DMCColourList[TweedList[i].ColourPairs[j+1].x].R - DMCColourList[TweedList[i].ColourPairs[j+1].y].R)
                                + (DMCColourList[TweedList[i].ColourPairs[j+1].x].G - DMCColourList[TweedList[i].ColourPairs[j+1].y].G)
                                * (DMCColourList[TweedList[i].ColourPairs[j+1].x].G - DMCColourList[TweedList[i].ColourPairs[j+1].y].G)
                                + (DMCColourList[TweedList[i].ColourPairs[j+1].x].B - DMCColourList[TweedList[i].ColourPairs[j+1].y].B)
                                * (DMCColourList[TweedList[i].ColourPairs[j+1].x].B - DMCColourList[TweedList[i].ColourPairs[j+1].y].B)
                                ;
                if (leastsquares2 < leastsquares1) {
                    swapped = true;
                    swap = TweedList[i].ColourPairs[j];
                    TweedList[i].ColourPairs[j] = TweedList[i].ColourPairs[j+1];
                    TweedList[i].ColourPairs[j+1] = swap;
                    }
                }
            }
        }

    Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::HelpButtonClick(TObject *Sender)
{
    AnsiString Address = ExtractFileDir(Application->ExeName) + "\\BackStitch.chm";
    AnsiString Location = "mk:@MSITStore:" + Address + "::/addthreadrange.html#start";
    ShellExecute(Application->Handle,"open",Location.c_str(),Location.c_str(),NULL,SW_SHOW);

}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::OKButtonClick(TObject *Sender)
{
    //save the range, and register it in the program's ini file but

    //first save the short title if not found
    TStringList *inputfile = new TStringList();
    if (!shorttitlefound) {
        AnsiString ThreadFileName = FileName;
        if (FileExists(ThreadFileName)) {
            inputfile->LoadFromFile(ThreadFileName);
            inputfile->Add("# DO NOT DELETE THIS LINE Short Title=" + ShortThreadName->Text);
            inputfile->SaveToFile(ThreadFileName);
            inputfile->Clear();
            }
        }
    delete inputfile;

    //save blended range
    AnsiString BlendFileName = ChangeFileExt(FileName,"Blend.csv");
    TStringList *outputfile = new TStringList();
    AnsiString text;

    //first lines
    outputfile->Add("# Blend Colours Generated by Chestnut Pens Charting Program for " + FileName);
    outputfile->Add("# WARNING: When Floss colours are updated, this file will not work");
    outputfile->Add("# Colour Component Names,Description,Red,Green,Blue,RGB Value,[pairs of indexes to colours in " + FileName + ", smallest difference first]");

    //write updated file to output file
    for (int i = 0; i < TweedList.Length; i++) {
        text = TweedList[i].Data.DMCColourNumber + ",";
        text = text + TweedList[i].Data.DMCColourName + ",";
        text = text + AnsiString(TweedList[i].Data.R) + ",";
        text = text + AnsiString(TweedList[i].Data.G) + ",";
        text = text + AnsiString(TweedList[i].Data.B) + ",";
        text = text + AnsiString(TweedList[i].Data.Colour) + ",";
        //text = text + TweedList[i].Data.AnchorColourNumber + ",";
        for (int j = 0; j < TweedList[i].ColourPairs.Length; j++) {
            text = text + AnsiString(TweedList[i].ColourPairs[j].x) + ",";
            text = text + AnsiString(TweedList[i].ColourPairs[j].y) + ",";
            }
        outputfile->Add(text);
        }

    outputfile->SaveToFile(BlendFileName);
    outputfile->Clear();

    delete outputfile;

    //now register the blend in the ini file
    TIniFile *IniFile = new TIniFile(ChangeFileExt(Application->ExeName, ".ini"));
    int numberofthreadranges = IniFile->ReadInteger("ThreadSettings","NumberofRanges",3);
    //AnsiString text;
    bool found = false;
    int pointer = 2;
    AnsiString interfilename = (ChangeFileExt(FileName,"").Trim()).LowerCase();
    if (numberofthreadranges != 3) {
        pointer = 3;
        while (!found && pointer < numberofthreadranges) {
            text = ((IniFile->ReadString("ThreadSettings","RangeFilename" + AnsiString(pointer),"DMCCottonFloss")).Trim()).LowerCase();
            if (text == interfilename) {
                found = true;
                pointer--;
                }
            pointer++;
            }
        }
    if (found && pointer > 2) {
        IniFile->WriteString("ThreadSettings","RangeTitle" + AnsiString(pointer),ThreadName->Text);
        } else {
        IniFile->WriteInteger("ThreadSettings","NumberofRanges",numberofthreadranges + 1);
        IniFile->WriteString("ThreadSettings","RangeTitle" + AnsiString(numberofthreadranges),ThreadName->Text);
        IniFile->WriteString("ThreadSettings","RangeFilename" + AnsiString(numberofthreadranges),ChangeFileExt(FileName,""));
        }
    delete IniFile;

    ModalResult = mrOk;

}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Image2MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    int arrayentry = X/10 + 15 * (Y/10);
    if (arrayentry < DMCColourList.Length) {
        //StatusBar1->Panels->Items[4]->Text = " DMC " + DMCColourList[arrayentry].DMCColourNumber + "  "
        Image2->Hint = " " + ShortThreadName->Text + " " + DMCColourList[arrayentry].DMCColourNumber + "  "
                                                     + DMCColourList[arrayentry].DMCColourName
                                                     + "  R:" + AnsiString(DMCColourList[arrayentry].R)
                                                     + "  G:" + AnsiString(DMCColourList[arrayentry].G)
                                                     + "  B:" + AnsiString(DMCColourList[arrayentry].B)
                                                     ;
        }

}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Image3MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    //find the blend colour the mouse is over
    int arrayentry = X/2 + 75 * (Y/2);
    if (arrayentry < TweedList.Length) {
        //StatusBar1->Panels->Items[4]->Text = " DMC " + TweedList[arrayentry].DMCColourNumber + "  "
        Image3->Hint = " " + ShortThreadName->Text + " " + TweedList[arrayentry].Data.DMCColourNumber + "  "
                                                     + TweedList[arrayentry].Data.DMCColourName
                                                     + "  R:" + AnsiString(TweedList[arrayentry].Data.R)
                                                     + "  G:" + AnsiString(TweedList[arrayentry].Data.G)
                                                     + "  B:" + AnsiString(TweedList[arrayentry].Data.B)
                                                     ;
        }
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Edit1Change(TObject *Sender)
{
    double value = ToDoubleDef(Edit1->Text,45.0);
    double oldvalue = value;
    if (value > 180.0) value = 180.0;
    if (value < 0.1) value = 0.1;
    if (value != oldvalue) Edit1->Text = AnsiString(value);
}
//---------------------------------------------------------------------------

void __fastcall TAddThreadRangeForm::Edit2Change(TObject *Sender)
{
    double value = ToDoubleDef(dynamic_cast <TEdit *>(Sender)->Text,0.26);
    double oldvalue = value;
    if (value > 1.0) value = 1.0;
    if (value < 0.0) value = 0.01;
    if (value != oldvalue) dynamic_cast <TEdit *>(Sender)->Text = AnsiString(value);

}
//---------------------------------------------------------------------------

