#include <vcl\vcl.h>
#pragma hdrstop

#include "MathsHelp.h"

//---------------------------------------------------------------------------

//trig functions in degrees not radians

double __fastcall cosD(double degrees)
{
    //tested & verified
		return cos(degrees * M_PI / 180.0);
}

double __fastcall sinD(double degrees)
{
    //tested & verified
		return sin(degrees * M_PI / 180.0);
}

double __fastcall tanD(double degrees)
{
    //tested & verified
		return tan(degrees * M_PI / 180.0);
}

double __fastcall acosD(double x)
{
    //tested & verified
		return (180.0 * acos(x)/M_PI) ;
}

double __fastcall asinD(double x)
{
    //tested & verified
		return (180.0 * asin(x)/M_PI) ;
}

double __fastcall atanD(double x)
{
    //tested & verified
		return (180.0 * atan(x)/M_PI) ;
}


//---------------------------------------------------------------------------

//Matrix and Vector methods
matrix __fastcall MatrixMultiply(matrix Matrix1, matrix Matrix2)
{
    //tested & verified
    matrix combined;

    //first column
    combined.aa = Matrix1.aa * Matrix2.aa + Matrix1.ba * Matrix2.ab + Matrix1.ca * Matrix2.ac;
    combined.ab = Matrix1.ab * Matrix2.aa + Matrix1.bb * Matrix2.ab + Matrix1.cb * Matrix2.ac;
    combined.ac = Matrix1.ac * Matrix2.aa + Matrix1.bc * Matrix2.ab + Matrix1.cc * Matrix2.ac;
    //second column
    combined.ba = Matrix1.aa * Matrix2.ba + Matrix1.ba * Matrix2.bb + Matrix1.ca * Matrix2.bc;
    combined.bb = Matrix1.ab * Matrix2.ba + Matrix1.bb * Matrix2.bb + Matrix1.cb * Matrix2.bc;
    combined.bc = Matrix1.ac * Matrix2.ba + Matrix1.bc * Matrix2.bb + Matrix1.cc * Matrix2.bc;
    //third column
    combined.ca = Matrix1.aa * Matrix2.ca + Matrix1.ba * Matrix2.cb + Matrix1.ca * Matrix2.cc;
    combined.cb = Matrix1.ab * Matrix2.ca + Matrix1.bb * Matrix2.cb + Matrix1.cb * Matrix2.cc;
    combined.cc = Matrix1.ac * Matrix2.ca + Matrix1.bc * Matrix2.cb + Matrix1.cc * Matrix2.cc;

    return combined;
}


vector3D __fastcall MatrixMultiply (matrix Matrix1, vector3D Vector1)
{
    //tested & verified
    vector3D combined;

    combined.x = Matrix1.aa * Vector1.x + Matrix1.ba * Vector1.y + Matrix1.ca * Vector1.z;
    combined.y = Matrix1.ab * Vector1.x + Matrix1.bb * Vector1.y + Matrix1.cb * Vector1.z;
    combined.z = Matrix1.ac * Vector1.x + Matrix1.bc * Vector1.y + Matrix1.cc * Vector1.z;


    return combined;

}

vector3D __fastcall UnitVector(vector3D vector1)
{
    //tested & verified
    //unitises the vector
    double length = vector1.x * vector1.x + vector1.y * vector1.y + vector1.z * vector1.z;
    length = sqrt(length);

    if (length) {
        vector1.x = vector1.x / length;
        vector1.y = vector1.y / length;
        vector1.z = vector1.z / length;
        } else {
        //there's a problem if we get here.
        vector1.x = 1.0;
        vector1.y = 0.0;
        vector1.z = 0.0;
        }

    return vector1;
}


vector3D __fastcall VectorProduct(vector3D vector1, vector3D vector2)
{
    //tested & verified
    //returns the vector product of two 3D vectors
    vector3D value;

    value.x = vector1.y * vector2.z - vector1.z * vector2.y;
    value.y = vector1.z * vector2.x - vector1.x * vector2.z;
    value.z = vector1.x * vector2.y - vector1.y * vector2.x;

    return value;

}


vector3D __fastcall ScaleVector(vector3D vector1,  double scalar)
{
    //tested & verified
    //returns the scaled vector

    vector1.x = vector1.x * scalar;
    vector1.y = vector1.y * scalar;
    vector1.z = vector1.z * scalar;

    return vector1;

}


double __fastcall ScalarProduct(vector3D vector1, vector3D vector2)
{
    //tested & verified
    //returns the scalar (dot) product of two 3D vectors
    double value;

    value = vector1.x * vector2.x + vector1.y * vector2.y + vector1.z * vector2.z;

    return value;

}


double __fastcall DistanceBetweenPoints(vector3D vector1, vector3D vector2)
{
    //tested & verified
    //distance between two nodal positions
    double distance = (vector1.x - vector2.x) * (vector1.x - vector2.x);

    distance += (vector1.y - vector2.y) * (vector1.y - vector2.y);
    distance += (vector1.z - vector2.z) * (vector1.z - vector2.z);

    if (distance > 0.0) {
        distance = sqrt(distance);
        } else {
        distance = 0.0;
        }

    return distance;

}


vector3D __fastcall AddVector(vector3D vector1, vector3D vector2)
{
    //tested & verified
    vector3D total;

    total.x = vector1.x + vector2.x;
    total.y = vector1.y + vector2.y;
    total.z = vector1.z + vector2.z;

    return total;
}


vector3D __fastcall SubtractVector(vector3D vector1, vector3D vector2)
{
    //tested & verified
    vector3D total;

    total.x = vector1.x - vector2.x;
    total.y = vector1.y - vector2.y;
    total.z = vector1.z - vector2.z;

    return total;
}


vector3D __fastcall PlaneNormal(vector3D vector1, vector3D vector2, vector3D vector3)
{
    //tested & verified. Results are slightly eratic, so use with caution as a normal calculator. It's best to use Vector Product
    //calculates the plane normal for a plane parallel to the one defined by the three points vector1, vector2, vector3
    vector3D planecoefficients;

    double XA = vector2.x + vector1.x;
    double XB = vector1.x + vector3.x;
    double XC = vector3.x + vector2.x;
    double YA = vector2.y + vector1.y;
    double YB = vector1.y + vector3.y;
    double YC = vector3.y + vector2.y;
    double ZA = vector2.z + vector1.z;
    double ZB = vector1.z + vector3.z;
    double ZC = vector3.z + vector2.z;

    planecoefficients.x = (vector2.y - vector1.y) * ZA + (vector1.y - vector3.y) * ZB + (vector3.y - vector2.y) * ZC;
    planecoefficients.y = (vector2.z - vector1.z) * XA + (vector1.z - vector3.z) * XB + (vector3.z - vector2.z) * XC;
    planecoefficients.z = (vector2.x - vector1.x) * YA + (vector1.x - vector3.x) * YB + (vector3.x - vector2.x) * YC;

    //the final co-efficient is not necessary, but is
    //D = -(planecoefficients.x * vector2.x + planecoefficients.y * vector2.y + planecoefficients.z * vector2.z)

    return planecoefficients;
}


plane __fastcall Plane(vector3D vector1, vector3D vector2, vector3D vector3)
{
    //tested & verified.
    plane planecoefficients;

    double XA = vector2.x + vector1.x;
    double XB = vector1.x + vector3.x;
    double XC = vector3.x + vector2.x;
    double YA = vector2.y + vector1.y;
    double YB = vector1.y + vector3.y;
    double YC = vector3.y + vector2.y;
    double ZA = vector2.z + vector1.z;
    double ZB = vector1.z + vector3.z;
    double ZC = vector3.z + vector2.z;

    planecoefficients.A = (vector2.y - vector1.y) * ZA + (vector1.y - vector3.y) * ZB + (vector3.y - vector2.y) * ZC;
    planecoefficients.B = (vector2.z - vector1.z) * XA + (vector1.z - vector3.z) * XB + (vector3.z - vector2.z) * XC;
    planecoefficients.C = (vector2.x - vector1.x) * YA + (vector1.x - vector3.x) * YB + (vector3.x - vector2.x) * YC;
    planecoefficients.D = -(planecoefficients.A * vector2.x + planecoefficients.B * vector2.y + planecoefficients.C * vector2.z);

    return planecoefficients;
}


bool __fastcall PointOnLine(vector3D LineStart,vector3D LineFinish,vector3D Point1)
{
    //tested & verified.
    //this method checks to see whether the vector Point1 is in line with the
    //2 nodes specifying the line.
    //This is checked by seeing whether the absolute value of the scalar product
    //is > 0.999999999999999 This gives an accuracy of 0.0000001 off the line & is reasonable.
    return (fabs(ScalarProduct(UnitVector(SubtractVector(LineFinish,LineStart)),UnitVector(SubtractVector(Point1,LineStart)))) > 0.999999999999999);
}


intersection3D __fastcall Intersection(plane Plane, vector3D LineStart, vector3D LineFinish)
{
    //tested & verified.
    //returns the position of the intersection between a plane and a line
    //u is the parameter 0 to 1 of the line. No checks are made to see that the
    //intersection parameter is between 0 and 1.

    intersection3D result;
    vector3D interresult;

    if ((Plane.A * (LineStart.x - LineFinish.x) + Plane.B * (LineStart.y - LineFinish.y) + Plane.C * (LineStart.z - LineFinish.z)) != 0.0) {
        double u = (Plane.A * LineStart.x + Plane.B * LineStart.y + Plane.C * LineStart.z + Plane.D)
                    /(Plane.A * (LineStart.x - LineFinish.x) + Plane.B * (LineStart.y - LineFinish.y) + Plane.C * (LineStart.z - LineFinish.z));

        interresult = AddVector(LineStart,ScaleVector(SubtractVector(LineFinish,LineStart),u));
        result.x = interresult.x;
        result.y = interresult.y;
        result.z = interresult.z;
        result.parameter = u;
        result.found = true;
        } else {
        result.x = 0.0;
        result.y = 0.0;
        result.z = 0.0;
        result.parameter = 0.0;
        result.found = false;
        }

    return result;
}


vector3D __fastcall CubicBezier(vector3D P1, vector3D P2, vector3D P3, vector3D P4, double t)
{
    //tested & verified.
	//calculates the cubic bezier result of 4 points, for parameter t through the curve. 0.0 <= t <= 1.0
	
	vector3D result;
	
	//set a default value
	//result.x = P1.x;
	//result.y = P1.y;
	//result.z = P1.z;
	result = P1;


	//bezier values are based on B(t) = Sum from i=0 to n of: Binomial(i,n) * (1-t)^n-i * t^i * Point i
	//This is specifically for cubic beziers.
	if (t >= 0.0 && t <= 1.0) {
		result = ScaleVector(P1,pow(1.0-t,3.0));
		result = AddVector(result,ScaleVector(P2,3.0 * pow(1.0-t,2.0) * t));
		result = AddVector(result,ScaleVector(P3,3.0 * (1.0-t) * t * t));
		result = AddVector(result,ScaleVector(P4,t * t * t));
		}
	
	return result;
}


//---------------------------------------------------------------------------


//misc functions

int __fastcall Heavyside(double value, double zeroaccuracy)
{
    //tested & verified
    //Heavyside function to normalize doubles into integers of value 1, 0 or -1
    //depending on whether above, at or below zero.
    int heavyside = 0;
    if (value > zeroaccuracy) {
        heavyside = 1;
        } else {
        if (value < -zeroaccuracy) {
            heavyside = -1;
            }
        }

    return heavyside;
}


double __fastcall ROUND(double value, int decimalplaces)
{
    //tested & verified
	//note, decimalplaces can be negative if say 1024.03 needs to be rounded to the nearest 1000 (ie decimalplaces = -3)
    double result;
	double interimvalue = value;
	int counter = decimalplaces;
	if (decimalplaces > 0) {
		while (counter != 0) {
			interimvalue = interimvalue * 10.0;
			counter--;
			}
		} else {
		if (decimalplaces < 0) {
			while (counter != 0) {
				interimvalue = interimvalue / 10.0;
				counter++;
				}
			}
		}
    if (floor(interimvalue) == floor(interimvalue + 0.5)) {
        result = floor(interimvalue);
        } else {
        result = floor(interimvalue + 0.5);
        }
	counter = decimalplaces;
	if (decimalplaces > 0) {
		while (counter != 0) {
			result = result / 10.0;
			counter--;
			}
		} else {
		if (decimalplaces < 0) {
			while (counter != 0) {
				result = result * 10.0;
				counter++;
				}
			}
		}
    return result;

}


int __fastcall Factorial(int value)
{
    //tested & verified
	int result = 1;
	int counter = 1;
	while (counter <= value) {
		result = result * counter;
		counter++;
		}
	
	return result;
}


int __fastcall Binomial(int entry, int order)
{
    //tested & verified
	//calculates the binomial factors eg 1,5,10,10,5,1 for an expansion of 5th power binomials
	//Binomial(0,5) gives 1, Binomial(1,5) gives 5, Binomial(2,5) gives 10, Binomial(3,5) gives 10,  Binomial(4,5) gives 5, Binomial(5,5) gives 1.
    int result = 1;
    if (entry > -1 && entry <= order) {
        result = (Factorial(order))/(Factorial(entry) * Factorial(order - entry));
        }
	return result;
    //return 1;
}


//---------------------------------------------------------------------------

