#ifndef MathsHelpH
#define MathsHelpH

#include <math.h>

//trig functions in degrees
double __fastcall cosD(double degrees);
double __fastcall sinD(double degrees);
double __fastcall tanD(double degrees);
double __fastcall acosD(double x);
double __fastcall asinD(double x);
double __fastcall atanD(double x);

//matrix methods and vector methods
typedef struct  {
    double aa, ba, ca;
    double ab, bb, cb;
    double ac, bc, cc;
    } matrix;
	
typedef struct  {
    double x, y, z;
    } vector3D;

typedef struct  {
    double x, y, z;
    double parameter;
    bool found;
    } intersection3D;

typedef struct {
	//defines a plane with equation Ax + By + Cz + D = 0;
	double A, B, C, D;
	} plane;

	
matrix __fastcall MatrixMultiply(matrix Matrix1, matrix Matrix2);
vector3D __fastcall MatrixMultiply(matrix Matrix1, vector3D Vector1);
vector3D __fastcall UnitVector(vector3D vector1);
vector3D __fastcall VectorProduct(vector3D vector1, vector3D vector2);
vector3D __fastcall ScaleVector(vector3D vector1, double scalar);
double __fastcall ScalarProduct(vector3D vector1, vector3D vector2);
double __fastcall DistanceBetweenPoints(vector3D vector1, vector3D vector2);
vector3D __fastcall AddVector(vector3D vector1, vector3D vector2);
vector3D __fastcall SubtractVector(vector3D vector1, vector3D vector2);
vector3D __fastcall PlaneNormal(vector3D vector1, vector3D vector2, vector3D vector3);
plane __fastcall Plane(vector3D vector1, vector3D vector2, vector3D vector3);
bool __fastcall PointOnLine(vector3D LineStart,vector3D LineFinish,vector3D Point1);
intersection3D __fastcall Intersection(plane Plane, vector3D LineStart, vector3D LineFinish);
vector3D __fastcall CubicBezier(vector3D P1, vector3D P2, vector3D P3, vector3D P4, double t);


//Various Miscellaneous functions
int __fastcall Heavyside(double value, double zeroaccuracy);
double __fastcall ROUND(double value, int decimalplaces);
int __fastcall Factorial(int value);
int __fastcall Binomial(int entry, int order);

#endif

