//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ObtainRGB.h"
#include "GraphicsHelp.h"
#include <clipbrd.hpp>
#include "StringHelp.h"
#include <shellapi.h>
#include "AddThreadRange.h"
#include "ColourEntry.h"
#include "StitchForm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TObtainRGBForm *ObtainRGBForm;
//---------------------------------------------------------------------------
__fastcall TObtainRGBForm::TObtainRGBForm(TComponent* Owner)
    : TForm(Owner)
{
    crDrawRectangle = (TCursor) (16);
    Screen->Cursors[crDrawRectangle] = LoadCursor((void *)HInstance, "DRAWRECTANGLE");
    crGrab = (TCursor) (15);
    Screen->Cursors[crGrab] = LoadCursor((void *)HInstance, "GRAB");
    crDropperCursor = (TCursor)(14);
    Screen->Cursors[crDropperCursor] = LoadCursor((void *)HInstance, "DROPPER");

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::FormCreate(TObject *Sender)
{
    Picture = new Graphics::TBitmap();
    ActionType = TNoAction;
    TopLeft.x = 0;
    TopLeft.y = 0;
    OldTopLeft.x = 0;
    OldTopLeft.y = 0;
    StartPointPicture.x = 0;
    StartPointPicture.y = 0;
    EndPointScreen.x = 0;
    EndPointScreen.y = 0;
    SquareSize = 1;
    dragging = false;
    wipedout = true;
    RegisteredThreads = false;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::FormDestroy(TObject *Sender)
{
    delete Picture;
    DMCColourList.Length = 0;
    UndoList.Length = 0;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::PasteImage1Click(TObject *Sender)
{
    //when an image is pasted into the program it goes here
    //Paste the bitmap on clipboard into the background image, but do an image conversion while doing so.
    PasteButton->ShowHint = false;
    TClipboard *pCB = Clipboard();
    if (pCB->HasFormat(CF_BITMAP)) {
        try {
            Picture->LoadFromClipboardFormat(CF_BITMAP, pCB->GetAsHandle(CF_BITMAP), 0);
            //Right, it has pasted
            //now to display
            TopLeft.x = 0;
            TopLeft.y = 0;
            SquareSize = 1;
            DisplayImage();
            Label8->Caption = "Zoom = " + AnsiString(SquareSize * 100) + "%";
            }

        catch (...) {
            ShowMessage("Error during Paste Operation");
            }
        }

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Edit1Click(TObject *Sender)
{
    PasteImage1->Enabled = (Clipboard()->HasFormat(CF_BITMAP));
    Undo2->Enabled = (bool)(UndoList.Length);

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::DisplayImage(void)
{
    //Display Picture in imagebox
    //this draws a copy of the bitmap held in memory on the image's canvas
    //int maxX = 0;
    //int maxY = 0;
    //first, clear picture
    Image1->Picture->Bitmap->Width = Image1->Width;
    Image1->Picture->Bitmap->Height = Image1->Height;
    TRect Rectangle;
    Image1->Canvas->Pen->Color = clBtnFace;
    Image1->Canvas->Brush->Color = clBtnFace;
    Rectangle.left = 0;
    Rectangle.right = Image1->Picture->Bitmap->Width;
    Rectangle.top = 0;
    Rectangle.bottom = Image1->Picture->Bitmap->Height;
    Image1->Canvas->FillRect(Rectangle);

    if (Picture->Width > 0 && Picture->Height > 0) {
        Rectangle.left = -TopLeft.x * SquareSize;
        Rectangle.top = -TopLeft.y * SquareSize;
        Rectangle.right = Rectangle.left + SquareSize * Picture->Width;
        Rectangle.bottom = Rectangle.top + SquareSize * Picture->Height;
        Image1->Canvas->StretchDraw(Rectangle,Picture);
        }

    wipedout = true;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::AppendColoursto1Click(TObject *Sender)
{
    if (OpenDialog1->Execute()) {
        SaveDialog1->FileName = OpenDialog1->FileName;
        AnsiString FileName = OpenDialog1->FileName;
        if (FileExists(FileName)) {
            TStringList *inputfile = new TStringList();
            inputfile->LoadFromFile(FileName);
            //interpret file
            //first delete all comment lines
            for (int i = 0; i < inputfile->Count; i++) {
                if (LeftString(inputfile->Strings[i],1) == "#") {
                    if (inputfile->Strings[i].Pos("Title=")){
                        ShortThreadName->Text = DeleteWord(inputfile->Strings[i],"=");
                        }
                    inputfile->Delete(i);
                    i--;
                    }
                }
            //now create colour array & fill it
            DMCColourList.Length = inputfile->Count;
            AnsiString text;
            int value;
            rgb RGB;
            hsb HSB;
            for (int i = 0; i < inputfile->Count; i++) {
                text = inputfile->Strings[i];
                text = text + ",";
                //Extract the DMC Colour number. This is a string as 'Ecru' has no numeric meaning!
                DMCColourList[i].DMCColourNumber = ExtractWord(text,",");
                text = DeleteWord(text,",");
                //Extract the DMC Colour Name
                DMCColourList[i].DMCColourName = ExtractWord(text,",");
                text = DeleteWord(text,",");
                DMCColourList[i].DrawWhite = true;
                //Now to decide if the individual colour components exceed 100, if so, draw the
                //glyph in black on the background, otherwise use white
                //The half-way value is 128, but printers tend to print colours a bit light
                //so can go below 128 and still read the black symbol
                value = ExtractWord(text,",").ToIntDef(0);
                DMCColourList[i].R = value;
                if (value > 100) DMCColourList[i].DrawWhite = false;
                text = DeleteWord(text,",");
                value = ExtractWord(text,",").ToIntDef(0);
                DMCColourList[i].G = value;
                if (value > 100) DMCColourList[i].DrawWhite = false;
                text = DeleteWord(text,",");
                value = ExtractWord(text,",").ToIntDef(0);
                DMCColourList[i].B = value;
                if (value > 100) DMCColourList[i].DrawWhite = false;
                text = DeleteWord(text,",");
                DMCColourList[i].Colour = (TColor) ExtractWord(text,",").ToIntDef(0);
                text = DeleteWord(text,",");
                DMCColourList[i].StitchCount = 0;
                DMCColourList[i].AnchorColourNumber = ExtractWord(text,",");
                DMCColourList[i].AnchorColourToUse = 0;
                DMCColourList[i].AnchorStitchCount = 0;
                RGB.R = DMCColourList[i].R;
                RGB.G = DMCColourList[i].G;
                RGB.B = DMCColourList[i].B;
                HSB = RGBtoHSB(RGB);
                DMCColourList[i].Hue = HSB.Hue;
                DMCColourList[i].Saturation = HSB.Saturation;
                DMCColourList[i].Brightness = HSB.Brightness;
                }
            //finish interpretation
            delete inputfile;
            }
        UpdateNumberofColours();
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::UpdateNumberofColours(void)
{
    Label4->Caption = "Number of Colours = " + AnsiString(DMCColourList.Length);
    ListBox1->Items->Clear();
    for (int i = 0; i < DMCColourList.Length; i++) {
        ListBox1->Items->Add(DMCColourList[i].DMCColourNumber + ","
                            + DMCColourList[i].DMCColourName + ","
                            + "R" + AnsiString(DMCColourList[i].R) + ","
                            + "G" + AnsiString(DMCColourList[i].G) + ","
                            + "B" + AnsiString(DMCColourList[i].B) + ","
                            );
        }
    ListBox1->ItemIndex = ListBox1->Items->Count - 1;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::SaveAs1Click(TObject *Sender)
{
    //save the colours
    bool save = SaveDialog1->Execute();
    bool overwrite = false;
    int entry = -1 ;
    //check to see if a registered file would be overwritten, and stop it.
    AnsiString text = ChangeFileExt(SaveDialog1->FileName.LowerCase(),"");
    TPlanForm *Planform = dynamic_cast <TPlanForm *>(Owner);
    for (int i = 0; i < Planform->ThreadFileNameList->Count; i++) {
        if (Planform->ThreadFileNameList->Strings[i].LowerCase() == text || Planform->ThreadFileNameList->Strings[i].LowerCase()+ "blend" == text) {
            save = false;
            overwrite = true;
            entry = i;
            }
        }
    if (save) {
        SaveThreads();
        Save1->Enabled = true;
        SaveButton->Enabled = true;
        RegisterDefinitionFile1->Enabled = true;
        } else {
        if (overwrite) {
            if (entry > -1 && entry < 3) {
                Application->MessageBox("Sorry, that would overwrite one of the Anchor, DMC or DMC Monochrome thread definitions supplied with the program.","SaveAs Aborted",IDOK | MB_ICONEXCLAMATION);
                } else {
                if (entry > 2) Application->MessageBox("Sorry, That would overwrite a registered thread file.","SaveAs Aborted",IDOK | MB_ICONEXCLAMATION);
                }
            Save1->Enabled = false;
            SaveButton->Enabled = false;
            RegisterDefinitionFile1->Enabled = false;
            }
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::SaveButtonClick(TObject *Sender)
{
    SaveButton->ShowHint = false;
    SaveThreads();

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::SaveThreads(void)
{
        AnsiString FileName = SaveDialog1->FileName;
        AnsiString text;
        TStringList *inputfile = new TStringList();


        if ((ShortThreadName->Text.Trim()).Length() ) {
            inputfile->Add("# " + ShortThreadName->Text + " Colours");
            inputfile->Add("# DO NOT DELETE THIS LINE Short Title=" + ShortThreadName->Text.Trim());
            }
        inputfile->Add("# Colour Definitions for the Chestnut Pens Cross Stitch Chart Creator Program");
        inputfile->Add("# ColourNumber,Description,Red,Green,Blue,RGB Value,Comment (ignored)");

        for (int i = 0; i < DMCColourList.Length; i++) {
            text = DMCColourList[i].DMCColourNumber + ","
                    + DMCColourList[i].DMCColourName + ","
                    + AnsiString(DMCColourList[i].R) + ","
                    + AnsiString(DMCColourList[i].G) + ","
                    + AnsiString(DMCColourList[i].B) + ","
                    + AnsiString(DMCColourList[i].Colour) + ","
                    + "Comment"
                    ;
            inputfile->Add(text);
            }

        inputfile->SaveToFile(FileName);
        delete inputfile;

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::RedChange(TObject *Sender)
{
    if (dynamic_cast <TEdit *>(Sender)) {
        int value = dynamic_cast <TEdit *>(Sender)->Text.ToIntDef(255);
        bool reset = false;
        if (value < 0) {
            value = 0;
            reset = true;
            }
        if (value > 255) {
            value = 255;
            reset = true;
            }
        if (reset) {
            dynamic_cast <TEdit *>(Sender)->OnChange = NULL;
            dynamic_cast <TEdit *>(Sender)->Text = AnsiString(value);
            dynamic_cast <TEdit *>(Sender)->OnChange = RedChange;
            }
        Shape1->Brush->Color = (TColor) (Red->Text.ToIntDef(255) + 256 * Green->Text.ToIntDef(255) + 256 * 256 * Blue->Text.ToIntDef(255));
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::PickRectangleClick(TObject *Sender)
{
    ActionType = TDrawRectangle;
    Image1->Cursor = crDrawRectangle;
    PickRectangle->ShowHint = false;
    PickRectangle->Down = true;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::PickEyeDropperClick(TObject *Sender)
{
    ActionType = TEyeDropper;
    Image1->Cursor = crDropperCursor;
    PickEyeDropper->ShowHint = false;
    PickEyeDropper->Down = true;

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::PanButtonClick(TObject *Sender)
{
    ActionType = TMoveItem;
    Image1->Cursor = crGrab;
    PanButton->ShowHint = false;
    PanButton->Down = true;

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Exit1Click(TObject *Sender)
{
    ModalResult = mrOk;    
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Image1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    chartposition Location = FindPicturePosition(X,Y);
    chartposition SP = FindImagePosition(Location.X,Location.Y);

    if (!Location.outofrange) {
        switch (ActionType) {
            case TMoveItem:         {
                                DisplayImage();
                                dragging = true;
                                OldTopLeft = TopLeft;
                                EndPointScreen.x = X;
                                EndPointScreen.y = Y;
                                }
                                break;
            case TDrawRectangle:    {
                                DisplayImage();
                                dragging = true;
                                if (SP.X != X || SP.Y != Y) {
                                    TRect Rectangle;
                                    Rectangle.left= SP.X;
                                    Rectangle.right = X;
                                    Rectangle.top = SP.Y;
                                    Rectangle.Bottom = Y;
                                    if (Rectangle.left > Rectangle.right) {
                                        int swap = Rectangle.left;
                                        Rectangle.left = Rectangle.right;
                                        Rectangle.right = swap;
                                        }
                                    if (Rectangle.top > Rectangle.bottom) {
                                        int swap = Rectangle.top;
                                        Rectangle.top = Rectangle.bottom;
                                        Rectangle.bottom = swap;
                                        }
                                    Image1->Canvas->DrawFocusRect(Rectangle);
                                    wipedout = false;
                                    }
                                StartPointPicture.x = Location.X;
                                StartPointPicture.y = Location.Y;
                                EndPointScreen.x = X;
                                EndPointScreen.y = Y;
                                }
                                break;
            case TEyeDropper:   {
                                TColor Colour;
                                Colour = Picture->Canvas->Pixels[Location.X][Location.Y];
                                rgb RGB = ExtractRGB(Colour);
                                Red->OnChange = NULL;
                                Green->OnChange = NULL;
                                Blue->OnChange = NULL;
                                Red->Text = AnsiString(RGB.R);
                                Green->Text = AnsiString(RGB.G);
                                Blue->Text = AnsiString(RGB.B);
                                Red->OnChange = RedChange;
                                Green->OnChange = RedChange;
                                Blue->OnChange = RedChange;
                                RedChange(Red);
                                }
                                break;
            default:            break;
            }
        }


}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    chartposition Location = FindPicturePosition(X,Y);
    chartposition SP;

    //if (!Location.outofrange) {
        switch (ActionType) {
            case TMoveItem:         {
                                if (dragging) {
                                    TopLeft.x = OldTopLeft.x - (X - EndPointScreen.x) / SquareSize;
                                    TopLeft.y = OldTopLeft.y - (Y - EndPointScreen.y) / SquareSize;
                                    if (TopLeft.x < 0) TopLeft.x = 0;
                                    if (TopLeft.y < 0) TopLeft.y = 0;
                                    if ((Picture->Width - TopLeft.x) < 2) {
                                        TopLeft.x = Picture->Width - 2;
                                        }
                                    if ((Picture->Height - TopLeft.y) < 2) {
                                        TopLeft.y = Picture->Height - 2;
                                        }
                                    DisplayImage();
                                    }
                                }
                                break;
            case TDrawRectangle:    {
                                if (dragging && !Location.outofrange) {
                                    TRect Rectangle;
                                    if (!wipedout) {
                                        SP = FindImagePosition(StartPointPicture.x,StartPointPicture.y);
                                        Rectangle.left= SP.X;
                                        Rectangle.right = EndPointScreen.x;
                                        Rectangle.top = SP.Y;
                                        Rectangle.Bottom = EndPointScreen.y;
                                        if (Rectangle.left > Rectangle.right) {
                                            int swap = Rectangle.left;
                                            Rectangle.left = Rectangle.right;
                                            Rectangle.right = swap;
                                            }
                                        if (Rectangle.top > Rectangle.bottom) {
                                            int swap = Rectangle.top;
                                            Rectangle.top = Rectangle.bottom;
                                            Rectangle.bottom = swap;
                                            }
                                        Image1->Canvas->DrawFocusRect(Rectangle);
                                        wipedout = true;
                                        }
                                    EndPointScreen.x = X;
                                    EndPointScreen.y = Y;
                                    SP = FindImagePosition(StartPointPicture.x,StartPointPicture.y);
                                    Rectangle.left= SP.X;
                                    Rectangle.right = EndPointScreen.x;
                                    Rectangle.top = SP.Y;
                                    Rectangle.Bottom = EndPointScreen.y;
                                    if (Rectangle.left > Rectangle.right) {
                                        int swap = Rectangle.left;
                                        Rectangle.left = Rectangle.right;
                                        Rectangle.right = swap;
                                        }
                                    if (Rectangle.top > Rectangle.bottom) {
                                        int swap = Rectangle.top;
                                        Rectangle.top = Rectangle.bottom;
                                        Rectangle.bottom = swap;
                                        }
                                    Image1->Canvas->DrawFocusRect(Rectangle);
                                    wipedout = false;
                                    }
                                }
                                break;
            default:            break;
            }
        //}

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Image1MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    chartposition Location = FindPicturePosition(X,Y);
    chartposition SP;

    //if (!Location.outofrange) {
        switch (ActionType) {
            case TMoveItem:         {
                                dragging = false;
                                }
                                break;
            case TDrawRectangle:    {
                                if (dragging && !Location.outofrange) {
                                    TRect Rectangle;
                                    if (!wipedout) {
                                        SP = FindImagePosition(StartPointPicture.x,StartPointPicture.y);
                                        Rectangle.left= SP.X;
                                        Rectangle.right = EndPointScreen.x;
                                        Rectangle.top = SP.Y;
                                        Rectangle.Bottom = EndPointScreen.y;
                                        if (Rectangle.left > Rectangle.right) {
                                            int swap = Rectangle.left;
                                            Rectangle.left = Rectangle.right;
                                            Rectangle.right = swap;
                                            }
                                        if (Rectangle.top > Rectangle.bottom) {
                                            int swap = Rectangle.top;
                                            Rectangle.top = Rectangle.bottom;
                                            Rectangle.bottom = swap;
                                            }
                                        Image1->Canvas->DrawFocusRect(Rectangle);
                                        wipedout = true;
                                        }
                                    dragging = false;
                                    //Right, that's the graphical bits done, now to do the calculation bits.
                                    int R = 0;
                                    int G = 0;
                                    int B = 0;
                                    int counter = 0;
                                    Rectangle.left = StartPointPicture.x;
                                    Rectangle.right = Location.X;
                                    Rectangle.top = StartPointPicture.y;
                                    Rectangle.bottom = Location.Y;
                                    if (Rectangle.left > Rectangle.right) {
                                        int swap = Rectangle.left;
                                        Rectangle.left = Rectangle.right;
                                        Rectangle.right = swap;
                                        }
                                    if (Rectangle.top > Rectangle.bottom) {
                                        int swap = Rectangle.top;
                                        Rectangle.top = Rectangle.bottom;
                                        Rectangle.bottom = swap;
                                        }
                                    Rectangle.bottom = Rectangle.bottom + 1;
                                    Rectangle.right = Rectangle.right + 1;
                                    rgb RGB;
                                    for (int i = Rectangle.left; i < Rectangle.right; i++) {
                                        for (int j = Rectangle.top; j < Rectangle.bottom; j++) {
                                            if (i > -1 && i < Picture->Width && j > -1 && j < Picture->Height && counter < 5000000) {
                                                RGB = ExtractRGB(Picture->Canvas->Pixels[i][j]);
                                                R += RGB.R;
                                                G += RGB.G;
                                                B += RGB.B;
                                                counter++;
                                                }
                                            }
                                        } //end 'for (i =
                                    if (counter > 0) {
                                        R = R/counter;
                                        G = G/counter;
                                        B = B/counter;
                                        } else {
                                        R = 0;
                                        G = 0;
                                        B = 0;
                                        }
                                    Red->OnChange = NULL;
                                    Green->OnChange = NULL;
                                    Blue->OnChange = NULL;
                                    Red->Text = AnsiString(R);
                                    Green->Text = AnsiString(G);
                                    Blue->Text = AnsiString(B);
                                    Red->OnChange = RedChange;
                                    Green->OnChange = RedChange;
                                    Blue->OnChange = RedChange;
                                    RedChange(Red);
                                    }
                                }
                                break;
            default:            break;
            }
        //}

}
//---------------------------------------------------------------------------

chartposition  __fastcall TObtainRGBForm::FindPicturePosition(int X, int Y)
{
    //This is an interesting problem. It takes the mouse position & converts
    //it into the location on the chart image.
    //The chart image may be scrolled & scaled in the image box.

    //Where the mouse is out of the chart image area, Location.outofrange is set to true.

    chartposition Location;
    Location.outofrange = false;
    Location.X = X;
    Location.Y = Y;

    //Transform to Picture co-ords
    Location.X = (TopLeft.x) + Location.X/(SquareSize);
    Location.Y = (TopLeft.y) + Location.Y/(SquareSize);

    //now check if within Picture limits.
    if (Location.X < 0
        || Location.X > Picture->Width
        || Location.Y < 0
        || Location.Y > Picture->Height) {
            Location.outofrange = true;
            }

    return Location;

}
//---------------------------------------------------------------------------

chartposition __fastcall TObtainRGBForm::FindImagePosition(int X, int Y)
{
    //Takes a chart position & transforms to an image position

    //Where the mouse is out of the image area, Location.outofrange is set to true.

    chartposition Location;
    Location.outofrange = false;
    Location.X = X;
    Location.Y = Y;

    //Transform to Picture co-ords
    /*
    Location.X = Location.X * SquareSize - TopLeft.x;
    Location.Y = Location.Y * SquareSize - TopLeft.y;
    */
    Location.X = (Location.X - TopLeft.x) * SquareSize;
    Location.Y = (Location.Y - TopLeft.y) * SquareSize;

    //now check if within Picture limits.
    if (Location.X < 0
        || Location.X > Image1->Width
        || Location.Y < 0
        || Location.Y > Image1->Height) {
            Location.outofrange = true;
            }

    return Location;

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::AddButtonClick(TObject *Sender)
{
    if ((ColourNumber->Text.Trim()).Length()) {
        if ((ColourName->Text.Trim()).Length()) {
            int index = 0;
            bool found = false;
            while (index < DMCColourList.Length && !found) {
                if ((ColourNumber->Text.Trim()).LowerCase() == (DMCColourList[index].DMCColourNumber).LowerCase()) {
                    found = true;
                    index--;
                    }
                index++;
                }
            DMCColourData DMCColour;
            if (found) {
                //for undo
                DMCColour = DMCColourList[index];
                } else {
                DMCColourList.Length++;
                }
            DMCColourList[index].DMCColourNumber = ColourNumber->Text.Trim();
            DMCColourList[index].DMCColourName = ColourName->Text.Trim();
            DMCColourList[index].R = Red->Text.ToIntDef(255);
            DMCColourList[index].G = Green->Text.ToIntDef(255);
            DMCColourList[index].B = Blue->Text.ToIntDef(255);
            DMCColourList[index].Colour = Shape1->Brush->Color;
            DMCColourList[index].DrawWhite = true;
            if (DMCColourList[index].R > 100) DMCColourList[index].DrawWhite = false;
            if (DMCColourList[index].G > 100) DMCColourList[index].DrawWhite = false;
            if (DMCColourList[index].B > 100) DMCColourList[index].DrawWhite = false;
            UpdateNumberofColours();
            if (found) {
                Label4->Caption = Label4->Caption + ". Entry for " + ColourNumber->Text.Trim() + " replaced.";
                //set up undo
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TReplaceItem;
                UndoList[UndoList.Length - 1].DMCColour = DMCColour;
                UndoList[UndoList.Length - 1].entry1 = index;
                } else {
                //set up undo
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TDeleteItem;
                UndoList[UndoList.Length - 1].entry1 = index;
                }
            Red->OnChange = NULL;
            Green->OnChange = NULL;
            Blue->OnChange = NULL;
            Red->Text = AnsiString(255);
            Green->Text = AnsiString(255);
            Blue->Text = AnsiString(255);
            Red->OnChange = RedChange;
            Green->OnChange = RedChange;
            Blue->OnChange = RedChange;
            RedChange(Red);
            } else {
            Application->MessageBox("Colour Name required","Error",IDOK);
            }
        } else {
        Application->MessageBox("Colour Number required","Error",IDOK);
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ColourNumberChange(TObject *Sender)
{
    if (dynamic_cast <TEdit *>(Sender)) {
        dynamic_cast <TEdit *>(Sender)->OnChange = NULL;
        dynamic_cast <TEdit *>(Sender)->Text = ReplaceChars(dynamic_cast <TEdit *>(Sender)->Text,","," ");
        dynamic_cast <TEdit *>(Sender)->OnChange = ColourNumberChange;
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ZoomInClick(TObject *Sender)
{
    if (SquareSize < 30) SquareSize++;
    DisplayImage();
    ZoomIn->ShowHint = false;
    Label8->Caption = "Zoom = " + AnsiString(SquareSize * 100) + "%";
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ZoomOutClick(TObject *Sender)
{
    if (SquareSize > 1) SquareSize--;
    DisplayImage();
    ZoomOut->ShowHint = false;
    Label8->Caption = "Zoom = " + AnsiString(SquareSize * 100) + "%";

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::FormMouseWheelDown(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
    static int c = 0;

    if (!(c%3)) {
        ZoomInClick(Sender);
        }
    c++;
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::FormMouseWheelUp(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
    static int c = 0;

    if (!(c%3)) {
        ZoomOutClick(Sender);
        }
    c++;

}
//---------------------------------------------------------------------------


void __fastcall TObtainRGBForm::RegisterDefinitionFile1Click(
      TObject *Sender)
{
    //starts the registering of this definition file
    bool saved = true;
    if (!Save1->Visible) {
        saved = SaveDialog1->Execute();
        }
    if (saved) {
        Save1->Enabled = true;
        SaveButton->Enabled = true;
        RegisterDefinitionFile1->Enabled = true;
        SaveThreads();

        TAddThreadRangeForm *AddThreadRangeForm = new TAddThreadRangeForm(this);

        AddThreadRangeForm->FileName = SaveDialog1->FileName;
        AddThreadRangeForm->Initialise();

        AddThreadRangeForm->ShowModal();

        if (AddThreadRangeForm->ModalResult == mrOk) {
            RegisteredThreads = true;
            }
        delete AddThreadRangeForm;

        if (RegisteredThreads) ModalResult = mrOk;
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Help2Click(TObject *Sender)
{
    AnsiString Address = ExtractFileDir(Application->ExeName) + "\\BackStitch.chm";
    AnsiString Location = "mk:@MSITStore:" + Address + "::/obtainrgb.html#start";
    ShellExecute(Application->Handle,"open",Location.c_str(),Location.c_str(),NULL,SW_SHOW);

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ListBox1DrawItem(TWinControl *Control,
      int Index, TRect &Rect, TOwnerDrawState State)
{
    // Draw coloured rectangle background & name of colour
    TRect Rectangle = Rect;
    TColor TempColour = ListBox1->Canvas->Brush->Color;
    ListBox1->Canvas->Brush->Style = bsSolid;
    if (ListBox1->Selected[Index]) {
        ListBox1->Canvas->Brush->Color = clHighlight;
        } else {
        ListBox1->Canvas->Brush->Color = DMCColourList[Index].Colour;
        }
    ListBox1->Canvas->FillRect(Rectangle);
    ListBox1->Canvas->Brush->Color = TempColour;
    if (ListBox1->Selected[Index]) {
        ListBox1->Canvas->Font->Color = clHighlightText;
        } else {
        if (DMCColourList[Index].DrawWhite) {
            ListBox1->Canvas->Font->Color = clWhite;
            } else {
            ListBox1->Canvas->Font->Color = clBlack;
            }
        }
    ListBox1->Canvas->Brush->Style = bsClear;
    //for some reason the next line doesn't work
    //ListBox1->Canvas->TextRect(Rectangle,1,1,ListBox1->Items->Strings[Index]);
    DrawText(ListBox1->Canvas->Handle,("  " + ListBox1->Items->Strings[Index]).c_str(),-1,(LPRECT) &Rectangle,DT_VCENTER | DT_SINGLELINE);
    ListBox1->Canvas->Brush->Style = bsSolid;

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ListBox1Click(TObject *Sender)
{
    ListBox1->Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::ListBox1DblClick(TObject *Sender)
{
    //Edit current entry
    bool found = false;
    int index = 0;
    while (!found && index < ListBox1->Items->Count) {
        if (ListBox1->Selected[index]) {
            found = true;
            //deselect all items now
            for (int i = 0; i < ListBox1->Items->Count; i++) {
                ListBox1->Selected[i] = false;
                }

            TColourEntryForm *ColourEntryForm = new TColourEntryForm(this);

            ColourEntryForm->ColourNumber->Text = DMCColourList[index].DMCColourNumber;
            ColourEntryForm->ColourName->Text = DMCColourList[index].DMCColourName;
            ColourEntryForm->Red->Text = AnsiString(DMCColourList[index].R);
            ColourEntryForm->Green->Text = AnsiString(DMCColourList[index].G);
            ColourEntryForm->Blue->Text = AnsiString(DMCColourList[index].B);
            //ColourEntryForm->RedChange(ColourEntryForm->Red);

            ColourEntryForm->ShowModal();

            if (ColourEntryForm->ModalResult == mrOk) {
                //set up undo
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TReplaceItem;
                UndoList[UndoList.Length - 1].DMCColour = DMCColourList[index];
                UndoList[UndoList.Length - 1].entry1 = index;
                //get the results back
                DMCColourList[index].DMCColourNumber = ColourEntryForm->ColourNumber->Text;
                DMCColourList[index].DMCColourName = ColourEntryForm->ColourName->Text;
                DMCColourList[index].R = ColourEntryForm->Red->Text.ToIntDef(255);
                DMCColourList[index].G = ColourEntryForm->Green->Text.ToIntDef(255);
                DMCColourList[index].B = ColourEntryForm->Blue->Text.ToIntDef(255);
                DMCColourList[index].Colour = (TColor)(DMCColourList[index].R + 256 * DMCColourList[index].G + 256 * 256 * DMCColourList[index].B);
                DMCColourList[index].DrawWhite = true;
                if (DMCColourList[index].R > 100 || DMCColourList[index].G > 100 || DMCColourList[index].B > 100) DMCColourList[index].DrawWhite = false;
                //show the result
                int TopIndex = ListBox1->TopIndex;
                UpdateNumberofColours();
                ListBox1->TopIndex = TopIndex;
                }

            delete ColourEntryForm;
            }
        index++;
        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::PopupMenu1Popup(TObject *Sender)
{
    Undo1->Enabled = (bool)(UndoList.Length);
    MoveUpOne1->Enabled = (ListBox1->SelCount == 1);
    MoveDownOne1->Enabled = MoveUpOne1->Enabled;
    Edit2->Enabled = MoveUpOne1->Enabled;
    Delete1->Enabled = (ListBox1->SelCount > 0);
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::MoveUpOne1Click(TObject *Sender)
{
    int index = 0;
    bool found = false;
    while (!found && index < ListBox1->Items->Count) {
        if (ListBox1->Selected[index]) {
            found = true;
            index--;
            }
        index++;
        }

    if (found && index > 0) {
        //set up undo
        UndoList.Length++;
        UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
        UndoList.Length++;
        UndoList[UndoList.Length - 1].undoaction = TSwapItems;
        UndoList[UndoList.Length - 1].entry1 = index - 1;
        UndoList[UndoList.Length - 1].entry2 = index;
        //not strictly needed, but is useful for the swap
        UndoList[UndoList.Length - 1].DMCColour = DMCColourList[index];

        //swap colours
        DMCColourList[index] = DMCColourList[index - 1];
        DMCColourList[index - 1] = UndoList[UndoList.Length - 1].DMCColour;
        //show the result
        int TopIndex = ListBox1->TopIndex;
        UpdateNumberofColours();
        ListBox1->TopIndex = TopIndex;
        }

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::MoveDownOne1Click(TObject *Sender)
{
    int index = 0;
    bool found = false;
    while (!found && index < ListBox1->Items->Count) {
        if (ListBox1->Selected[index]) {
            found = true;
            index--;
            }
        index++;
        }

    if (found && index < DMCColourList.Length - 1) {
        //set up undo
        UndoList.Length++;
        UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
        UndoList.Length++;
        UndoList[UndoList.Length - 1].undoaction = TSwapItems;
        UndoList[UndoList.Length - 1].entry1 = index + 1;
        UndoList[UndoList.Length - 1].entry2 = index;
        //not strictly needed, but is useful for the swap
        UndoList[UndoList.Length - 1].DMCColour = DMCColourList[index];

        //swap colours
        DMCColourList[index] = DMCColourList[index + 1];
        DMCColourList[index + 1] = UndoList[UndoList.Length - 1].DMCColour;
        //show the result
        int TopIndex = ListBox1->TopIndex;
        UpdateNumberofColours();
        ListBox1->TopIndex = TopIndex;
        }

}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Delete1Click(TObject *Sender)
{
    if (ListBox1->SelCount > 0) {
        //set up undo
        UndoList.Length++;
        UndoList[UndoList.Length - 1].undoaction = TEndofUndo;
        int count = 0;

        for (int i = ListBox1->Items->Count - 1; i > -1; i--) {
            if (ListBox1->Selected[i]) {
                //undo
                UndoList.Length++;
                UndoList[UndoList.Length - 1].undoaction = TInsertItem;
                UndoList[UndoList.Length - 1].entry1 = i;
                UndoList[UndoList.Length - 1].DMCColour = DMCColourList[i];
                //do
                for (int j= i; j < DMCColourList.Length - 1; j++) {
                    DMCColourList[j] = DMCColourList[j + 1];
                    }
                DMCColourList.Length--;
                count++;
                }
            }
        int TopIndex = ListBox1->TopIndex - count;
        UpdateNumberofColours();
        ListBox1->TopIndex = TopIndex;

        }
}
//---------------------------------------------------------------------------

void __fastcall TObtainRGBForm::Undo1Click(TObject *Sender)
{
    //carries out all the undo actions
    if (UndoList.Length) {
        bool undoendfound = false;

        while (!undoendfound && UndoList.Length) {
            switch (UndoList[UndoList.Length - 1].undoaction) {
                case TReplaceItem:  {
                                    //Replace the colour
                                    DMCColourList[UndoList[UndoList.Length - 1].entry1] = UndoList[UndoList.Length - 1].DMCColour;
                                    }
                                    break;
                case TInsertItem:   {
                                    //Insert a colour
                                    DMCColourList.Length++;
                                    for (int i = DMCColourList.Length - 1; i > UndoList[UndoList.Length - 1].entry1; i--) {
                                        DMCColourList[i] = DMCColourList[i-1];
                                        }
                                    DMCColourList[UndoList[UndoList.Length - 1].entry1] = UndoList[UndoList.Length - 1].DMCColour;
                                    }
                                    break;
                case TDeleteItem:   {
                                    //Delete the colour
                                    for (int i = UndoList[UndoList.Length - 1].entry1; i < DMCColourList.Length - 1; i++) {
                                        DMCColourList[i] = DMCColourList[i+1];
                                        }
                                    DMCColourList.Length--;
                                    }
                                    break;
                case TSwapItems:    {
                                    //Swap two colours
                                    DMCColourData Swap = DMCColourList[UndoList[UndoList.Length - 1].entry1];
                                    DMCColourList[UndoList[UndoList.Length - 1].entry1] = DMCColourList[UndoList[UndoList.Length - 1].entry2];
                                    DMCColourList[UndoList[UndoList.Length - 1].entry2] = Swap;
                                    }
                                    break;
                default:            {
                                    //Finish undo
                                    undoendfound = true;
                                    }
                                    break;
                }
            UndoList.Length--;
            }
            
        int TopIndex = ListBox1->TopIndex;
        UpdateNumberofColours();
        ListBox1->TopIndex = TopIndex;
        }
}
//---------------------------------------------------------------------------


