//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "PrintPreview.h"
#include <clipbrd.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPrintPreviewForm *PrintPreviewForm;
//---------------------------------------------------------------------------
__fastcall TPrintPreviewForm::TPrintPreviewForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TPrintPreviewForm::FormCreate(TObject *Sender)
{
    PageList = new TList();
    TopLeftCorner.x = 0;
    TopLeftCorner.y = 0;
    MagScale = 1;
    ShrinkScale = 1;
    PageNumber = 0;
    dragging = false;
}
//---------------------------------------------------------------------------
void __fastcall TPrintPreviewForm::FormDestroy(TObject *Sender)
{
    //delete all graphics saved in the undo array
    for (int i = 0; i < PageList->Count; i++) {
        Graphics::TBitmap *TempPicture =  (Graphics::TBitmap *) PageList->Items[i];
        delete TempPicture;
        }
    delete PageList;

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::DisplayCurrentPage(void)
{
    if (PageNumber < PageList->Count) {
        Image1->Picture->Bitmap->Width = Image1->Width;
        Image1->Picture->Bitmap->Height = Image1->Height;
        Image1->Picture->Bitmap->Canvas->Pen->Color = clBackground;
        Image1->Picture->Bitmap->Canvas->Pen->Style = psSolid;
        Image1->Picture->Bitmap->Canvas->Brush->Color = clBackground;
        Image1->Picture->Bitmap->Canvas->Brush->Style = bsSolid;
        Image1->Picture->Bitmap->Canvas->Rectangle(0,0,Image1->Picture->Bitmap->Width,Image1->Picture->Bitmap->Height);

        Graphics::TBitmap *TempPicture =  (Graphics::TBitmap *) PageList->Items[PageNumber];
        TRect Rectangle;
        if (MagScale > 1) {
            Rectangle.Left = - TopLeftCorner.x * MagScale;
            Rectangle.Top = - TopLeftCorner.y * MagScale;
            Rectangle.Right = Rectangle.Left + TempPicture->Width * MagScale;
            Rectangle.Bottom = Rectangle.Top + TempPicture->Height * MagScale;
            } else {
            if (ShrinkScale == 0) ShrinkScale = 1;
            Rectangle.Left = - TopLeftCorner.x / ShrinkScale;
            Rectangle.Top = - TopLeftCorner.y / ShrinkScale;
            Rectangle.Right = Rectangle.Left + TempPicture->Width / ShrinkScale;
            Rectangle.Bottom = Rectangle.Top + TempPicture->Height / ShrinkScale;
            }
        Image1->Picture->Bitmap->Canvas->StretchDraw(Rectangle,TempPicture);
        Label1->Caption = " Scale " + AnsiString(MagScale) + " : " + AnsiString(ShrinkScale);
        Label2->Caption = "of " + AnsiString(PageList->Count);
        PageNumberEdit->Text = AnsiString(PageNumber + 1);
        }
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::CreatePages(void)
{
    for (int i = 0; i < PageList->Capacity; i++) {
        Graphics::TBitmap *TempPicture = new Graphics::TBitmap();
        TempPicture->Width = PageSize.x;
        TempPicture->Height = PageSize.y;
        //White Background
        TempPicture->Canvas->Pen->Color = clWhite;
        TempPicture->Canvas->Pen->Style = psSolid;
        TempPicture->Canvas->Brush->Color = clWhite;
        TempPicture->Canvas->Brush->Style = bsSolid;
        TempPicture->Canvas->Rectangle(0,0,PageSize.x,PageSize.y);
        PageList->Add(TempPicture);
        }

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::FormResize(TObject *Sender)
{
    Image1->Picture->Bitmap->Width = Image1->Width;
    Image1->Picture->Bitmap->Height = Image1->Height;
    DisplayCurrentPage();
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::FormMouseWheelUp(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
    static int count = 0;
    count++;
    if (!(count % 3)) {
        if (ShrinkScale == 1) {
            if (MagScale < 30) MagScale++;
            } else {
            if (ShrinkScale < 1) ShrinkScale = 2;
            ShrinkScale--;
            }
        DisplayCurrentPage();
        }
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::FormMouseWheelDown(TObject *Sender,
      TShiftState Shift, TPoint &MousePos, bool &Handled)
{
    static int count = 0;
    count++;
    if (!(count % 3)) {
        if (MagScale == 1) {
            if (ShrinkScale < 30) ShrinkScale++;
            } else {
            if (MagScale < 1) MagScale = 2;
            MagScale--;
            }
        DisplayCurrentPage();
        }

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::PageNumberEditKeyPress(TObject *Sender,
      char &Key)
{

    if (Key == '\r') {
        PageNumber = PageNumberEdit->Text.ToIntDef(1) - 1;
        if (PageNumber < 0 ) PageNumber = 0;
        if (PageNumber > PageList->Count - 1) PageNumber = PageList->Count;
        DisplayCurrentPage();
        }

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::NextBtnClick(TObject *Sender)
{
    PageNumber++;
    if (PageNumber == PageList->Count) PageNumber = 0;
    DisplayCurrentPage();
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::PreviousBtnClick(TObject *Sender)
{
    PageNumber--;
    if (PageNumber == -1) PageNumber = PageList->Count - 1;
    DisplayCurrentPage();

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::FirstBtn1Click(TObject *Sender)
{
    PageNumber = 0;
    DisplayCurrentPage();
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::LastBtnClick(TObject *Sender)
{
    PageNumber = PageList->Count - 1;
    DisplayCurrentPage();

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::ResetButtonClick(TObject *Sender)
{
    TopLeftCorner.x = 0;
    TopLeftCorner.y = 0;
    MagScale = 1;
    ShrinkScale = 1;
    DisplayCurrentPage();

}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::Image1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    dragging = true;
    OldXY.x = X;
    OldXY.y = Y;
    OldTopLeftCorner = TopLeftCorner;
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::Image1MouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    dragging = false;    
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    if (dragging) {
        if (MagScale == 1) {
            TopLeftCorner.x = OldTopLeftCorner.x - (X - OldXY.x) * ShrinkScale;
            TopLeftCorner.y = OldTopLeftCorner.y - (Y - OldXY.y) * ShrinkScale;
            } else {
            TopLeftCorner.x = OldTopLeftCorner.x - (X - OldXY.x) / MagScale;
            TopLeftCorner.y = OldTopLeftCorner.y - (Y - OldXY.y) / MagScale;
            }
        }
    DisplayCurrentPage();
}
//---------------------------------------------------------------------------

void __fastcall TPrintPreviewForm::CopyPagetoClipboard1Click(
      TObject *Sender)
{
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    Graphics::TBitmap *Bitmap = (Graphics::TBitmap *) PageList->Items[PageNumber];
    try {
        // generate a clipboard format, with data and palette
        // save the data to the clipboard using that format and
        // the generated data
        Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
        Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
        }

    catch (...) {
        ShowMessage("Sorry, Failed to copy image to clipboard.");
        }


}
//---------------------------------------------------------------------------

