//---------------------------------------------------------------------------
#ifndef StitchStructuresH
#define StitchStructuresH

//---------------------------------------------------------------------------

typedef struct {
    AnsiString DMCColourNumber;
    AnsiString DMCColourName;
    AnsiString AnchorColourNumber;
    TColor Colour;
    bool DrawWhite;
    int StitchCount;
    int AnchorColourToUse;
    int AnchorStitchCount;
    int AnchorGlyphToUse;
    int R, G, B;
    double Hue, Saturation, Brightness;
    } DMCColourData;
//---------------------------------------------------------------------------

typedef struct {
    AnsiString DMCColourName;
    TColor Colour;
    bool DrawWhite;
    int StitchCount;
    } ColourData;

//---------------------------------------------------------------------------

typedef struct {
    float X1;
    float Y1;
    float X2;
    float Y2;
    int ColourNumber;
    TColor Colour;
    } BackStitchData;

//---------------------------------------------------------------------------

typedef struct {
    int offsetx, offsety;
    double scalefactor;
    AnsiString filename;
    bool loaded,saved;
    } originalimagefile;

//---------------------------------------------------------------------------

typedef struct {
    double X, Y;
    bool outofrange;
    } mouseposition;

//---------------------------------------------------------------------------

typedef struct {
    int X, Y;
    bool outofrange;
    } chartposition;

//---------------------------------------------------------------------------

typedef struct {
    int startX, startY, endX, endY;
    double startchartX, startchartY, endchartX, endchartY;
    } rubberband;

//---------------------------------------------------------------------------

typedef struct {
    int X, Y;
    TColor Colour;
    } pixeleditundo;

//---------------------------------------------------------------------------

typedef struct {
    bool FrontSheet,IndexPage,ColourChart,MonochromeChart,DetailedKey,BackStitches,StitchesPerPage,FabricDetails,ChartPageXRef;
    int TitleSize,ImageCopyrightSize,ChartCopyrightSize,ChartFontSize;
    AnsiString Title,TitleFont,ImageCopyright,ImageCopyrightFont,ChartCopyright,ChartCopyrightFont,ChartFont;
    } printout;

//---------------------------------------------------------------------------

typedef struct {
    TColor Original, Used;
    } coloursub;
    
//---------------------------------------------------------------------------

typedef struct {
    int colourspace,imageheight,imagewidth,dithersetting;
    bool dither;
    double RGBOverall,R,G,B,HSBOverall,H,S,Br;
    } threadconvsettings;
    
//---------------------------------------------------------------------------

typedef struct {
    int leastsquares, entrynumber;
    } nearestcolour;

//---------------------------------------------------------------------------

//                     0                1                2                   3                 4            5            6          7             8         9           10          11         12           13                      14               15                16                 17                  18                19                20                21             22             23               24         25             26              27                      28          29              30              31                  32              33           34               35
enum TThreadSortOrder {TTSOStitchCount, TTSODMCNumber  , TTSOHue};
enum TLassoType       {TNoLasso       , TRectLasso     , TFlexLasso};
enum TBlendType       {TDMCColour     , TBlendedColour , TNoThread};
enum TConfettiLevel   {TLowConfetti   , TMediumConfetti, THighConfetti, TUnstitchable};
enum TActionType      {TNoAction      , TPixelEdit     , TDrawLine    , TFloodFill   , TDrawRectangle, TDrawCircle, TEyeDropper, TSelectItem, TTextItem, TEraseItem, TBlockItem, TCopyItem, TMoveItem, TPickFloodFillColour, TPickTextColour, TPickLineColour, TPickRectangleColour, TPickCircleColour, TPickPointColour, TEditPointPoints, TEditBlockPoints, TEditLineStart, TEditLineEnd, TEditErasePoints, TBlockArray, TRotateItem, TMirrorItem, TChangeRectangleSize, TChangeCircleSize, TDrawPolyLine, TDrawFreeForm, TPickPolyLineColour, TEditPolyLine, TEditFreeForm, TPasteBlock, TPickFreeFormColour};
//---------------------------------------------------------------------------

typedef struct {
    DMCColourData Data;
    DynamicArray <TPoint> ColourPairs;
    } TweedColourData;

//---------------------------------------------------------------------------

typedef struct {
    TBlendType type;
    AnsiString ColourName, ColourNumber;
    int arrayref;
    } BlendXRef;

//---------------------------------------------------------------------------

typedef struct {
    int refno, stitches, DMCArrayRef;
    bool used;
    } usedcolour;

//---------------------------------------------------------------------------

typedef struct {
    int maxcolours, mincolours;
    double average;
    TPoint Max, Min;
    } colourcomplexity;


//structures for the Image Editor

//                     0                1                2                  3             4                 5             6                  7                  8               9              10
enum TDisplayType {TDisplayNone       , TDisplayPoint  , TDisplayRectangle, TDisplayLine, TDisplayFreeForm, TDisplayText, TDisplayFloodFill, TDisplayEraseItem, TDisplayCircle, TDisplayBlock, TDisplayPolyLine};
enum TAntiAlias   {TAANone            , TAADMC         , TAABlend         } ;
enum TMirrorAxis  {TMAHorizontal      , TMAVertical    , TMANone          } ;

//all angles are in degrees multiplied by 10. So Angle = 3599 is actually 359.9 degrees.

typedef struct {
    int ItemNumber;
    TDisplayType DisplayType;
    TRect Area;
    AnsiString UndoName;
    bool selected;
    bool visible;
    } displaylist;

typedef struct {
    int ItemNumber;
    TDisplayType DisplayType;
    int DisplayXRef;
    int EntityXRef;
    } xref;

typedef struct {
    int ItemNumber;
    TColor Colour;
    DynamicArray <TPoint> Points;
    } pointitem;

typedef struct {
    int x,y;
    TColor Colour;
    } pixel;

typedef struct {
    int x,y,originalx,originaly;
    TColor Colour;
    bool transparent;
    } transpixel;

typedef struct {
    int ItemNumber;
    DynamicArray <pixel> Points;
    } eraseitem;

typedef struct {
    int ItemNumber;
    TColor Colour;
    int LineWidth;
    TPoint StartPoint;
    TPoint EndPoint;
    DynamicArray <pixel> Points;
    } lineitem;

typedef struct {
    int ItemNumber;
    TColor PenColour;
    TColor BrushColour;
    int LineWidth;
    TRect Area;
    TShapeType Shape;
    //angle in tenths of a degree
    int Angle;
    bool Filled;
    DynamicArray <pixel> Points;
    } stdshapeitem;

typedef struct {
    int ItemNumber;
    TRect Area;
    bool Linked;
    int LinkedBlock;
    TPoint RotationCentre;
    int Angle;
    TPoint MirrorCentre;
    TMirrorAxis Axis;
    TPoint delta;
    DynamicArray <transpixel> Points;
    } blockitem;

typedef struct {
    int ItemNumber;
    TColor PenColour;
    TColor BrushColour;
    int LineWidth;
    bool closed;
    bool filled;
    DynamicArray <pixel> Points;
    DynamicArray <TPoint> Corners;
    } freeformshapeitem;

typedef struct {
    int ItemNumber;
    AnsiString Text;
    AnsiString Name;
    //note Height is -ve when no leading is used above chars, the -ve sign is added automatically
    int Height;
    bool Bold;
    bool Italic;
    bool Underlined;
    bool Strikeout;
    TColor Colour;
    //angle is in tenths of a degree
    int Angle;
    TPoint StartPoint;
    DynamicArray <pixel> Points;
    } textitem;

typedef struct {
    int ItemNumber;
    TColor Colour;
    TColor FilledColour;
    TPoint StartPoint;
    DynamicArray <TPoint> Points;
    } floodfillitem;

typedef struct {
    int ItemNumber;
    bool ismaster;
    int numberofslaves;
    int EntityXRef;
    int DisplayXRef;
    int masterblock;
    } blockquery;

typedef struct {
    int x,y;
    int errorsquared;
    } blockcentresearch;

/*typedef struct {
    int newentry;
    TColor Colour;
    } coloursort;*/

enum TUndoType   {TStartUndo, TThreadListEntry, TSymbolListEntry} ;
enum TUndoAction {TEndofUndo, TSwapItems      , TDeleteItem     , TInsertItem, TReplaceItem};

typedef struct {
    TUndoType undotype;
    int row, value;
    } undolist;

typedef struct {
    int threadnumber, threadbrightness,symbolnumber;
    } threadbrightnesssort;

typedef struct {
    int symbolnumber, darkpixels;
    } symboldarknesssort;

typedef struct {
    int arrayentry;
    AnsiString filename;
    AnsiString description;
    } threadfile;

typedef struct {
    TUndoAction undoaction;
    DMCColourData DMCColour;
    int entry1, entry2;
    } colourdefineundo;


#endif
