#include <vcl\vcl.h>
#pragma hdrstop

#include "StringHelp.h"
#include <math.h>

//AnsiString Helper functions
//These will be slowly added to as the need arises.
//In the mean time only the prototype need remain fixed.


float __fastcall ToFloatDef(AnsiString Value, float DefaultVal)
{
    //equivalent to the .ToIntDef(value) function for doubles
    try {
        return (float)Value.ToDouble();
        }
    catch (...)
        {
        return DefaultVal;
        }
}
//---------------------------------------------------------------------------


double __fastcall ToDoubleDef(AnsiString Value, double DefaultVal)
{
    //equivalent to the .ToIntDef(value) function for doubles
    try {
        if (Value.Length()) {
            return Value.ToDouble();
            } else {
            return DefaultVal;
            }
        }
    catch (...)
        {
        //try a bit harder. Go through and check every character is within a list of
        //acceptable characters
        AnsiString temp;
        //char acceptable[] = "1234567890.eE+-";
        AnsiString acceptable = "1234567890.eE+-";
        int linelength = Value.Length();
        for (int a = 1 ; a < linelength + 1; a++) {
            //if (isprint(Value[a])) temp += AnsiString(Value[a]);

            //if (strchr(acceptable,Value[a])) temp += AnsiString(Value[a]);
            if (acceptable.Pos(Value.SubString(a,1))) temp += AnsiString(Value[a]);
            }

            
        try {
            return temp.ToDouble();
            }

        catch (...)
            {
            return DefaultVal;
            }
        }
}
//---------------------------------------------------------------------------




AnsiString __fastcall FixedLenFloat(int Length, long double value, TFloatFormat format, int precision, int digits)
{
    //does the same as FloatToStrF, only using extra figure "Length"
    //to indicate how long the string should be. If the FloatToStrF figure returns
    //a string longer than Length, no error is flagged.
    AnsiString string = FloatToStrF(value, format, precision, digits);
    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall FixedLenInt(int Length, int value)
{
    //returns a fixed length string with length "Length" and value "value"
    AnsiString string = AnsiString(value);
    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall FixedLenString(int Length, AnsiString string)
{
    //Returns the string padded out with spaces
    while (string.Length() < Length) string = " " + string;

    return string;

}
//---------------------------------------------------------------------------




AnsiString __fastcall FixedLen(int Length, bool value)
{
    AnsiString string;
    if (value) {
        string = "true";
        } else {
        string = "false";
        }
    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall FixedLen(int Length, int value)
{
    //returns a fixed length string with length "Length" and value "value"
    AnsiString string = AnsiString(value);
    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall FixedLen(int Length, long double value, TFloatFormat format, int precision, int digits)
{
    //does the same as FloatToStrF, only using extra figure "Length"
    //to indicate how long the string should be. If the FloatToStrF figure returns
    //a string longer than Length, no error is flagged.
    AnsiString string = FloatToStrF(value, format, precision, digits);
    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall FixedLen(int Length, AnsiString string)
{
    //Returns the string padded out with spaces
    while (string.Length() < Length) string = " " + string;

    return string;

}
//---------------------------------------------------------------------------



AnsiString __fastcall ForceLen(int Length, long double value, TFloatFormat format, int precision, int digits)
{
    //does the same as FloatToStrF, only using extra figure "Length"
    //to indicate how long the string should be. If the FloatToStrF figure returns
    //a string longer than Length, no error is flagged - but some efforts are
    //made to make sure the length is correct. Also requires there to be a
    //decimal point
    int Precision = precision;
    int Digits = digits;

    AnsiString string = FloatToStrF(value, format, Precision, Digits);

    while (string.Length() > Length && Digits > 0) {
        Digits--;
        string = FloatToStrF(value, format, Precision, Digits);
        }
    while (string.Length() > Length && Precision > 0) {
        Precision--;
        string = FloatToStrF(value, format, Precision, Digits);
        }
    if(!string.Pos(".")) {
        if (string.Length() < Length) {
            string += ".";
            } else {
            if (format != ffExponent) string = ForceLen(Length,value,ffExponent,precision,digits);
            }
        }

    while (string.Length() < Length) string = " " + string;

    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall FixedLenZeroPadded(int Length, bool value)
{
    //returns a fixed length string with length "Length" and value "value"
    //padded out with zeros.
    AnsiString string = AnsiString((int)value);
    while (string.Length() < Length) string = "0" + string;

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall FixedLenZeroPadded(int Length, int value)
{
    //returns a fixed length string with length "Length" and value "value"
    //padded out with zeros.
    AnsiString string = AnsiString(value);
    while (string.Length() < Length) string = "0" + string;

    return string;

}
//---------------------------------------------------------------------------




AnsiString __fastcall Pad(int Length, bool value)
{
    //Returns the string padded out with spaces
    //similar to FixedLen, only the padding occurs after
    //the word.
    AnsiString string;
    if (value) {
        string = "true";
        } else {
        string = "false";
        }
    while (string.Length() < Length) string += " ";

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall Pad(int Length, int value)
{
    //Returns the string padded out with spaces
    //similar to FixedLen, only the padding occurs after
    //the word.
    AnsiString string = AnsiString(value);
    while (string.Length() < Length) string += " ";

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall Pad(int Length, long double value, TFloatFormat format, int precision, int digits)
{
    //does the same as FloatToStrF, only using extra figure "Length"
    //to indicate how long the string should be. If the FloatToStrF figure returns
    //a string longer than Length, no error is flagged.
    AnsiString string = FloatToStrF(value, format, precision, digits);
    while (string.Length() < Length) string += " ";

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall Pad(int Length, AnsiString string)
{
    //Returns the string padded out with spaces
    //similar to FixedLenString, only the padding occurs after
    //the word.
    while (string.Length() < Length) string += " ";

    return string;

}
//---------------------------------------------------------------------------




AnsiString __fastcall ExtractWord(AnsiString string, AnsiString Delimiter)
{
    if (string.Length() != 0) {
        //this function extracts the next word based on the string and the delimiter
        while (string.SubString(1,1) == Delimiter) string.Delete(1,1);
        int a = string.Pos(Delimiter);
        if (a) {
            string = string.SubString(1,a - 1);
            } else {
            string = "";
            }
        }
    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall DeleteWord(AnsiString string, AnsiString word)
{
    if (string.Length()) {
        //removes up to and including the first occurrence of word in string
        int a = string.Pos(word);
        if (a) {
            string.Delete(1,a + word.Length() - 1);
            }
        }
    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall RemoveWord(AnsiString string, AnsiString word)
{
    if (string.Length()) {
        //similar to DeleteWord
        //removes the first occurrence of word in string
        int a = string.Pos(word);
        if (a) {
            string.Delete(a,word.Length());
            }
        }
    return string;
}
//---------------------------------------------------------------------------


AnsiString __fastcall FixedLenIntZeroPadded(int Length, int value)
{
    //returns a fixed length string with length "Length" and value "value"
    //padded out with zeros.
    AnsiString string = AnsiString(value);
    while (string.Length() < Length) string = "0" + string;

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall RemoveCommas(AnsiString string)
{
    int i;
    while (string.Pos(",")) {
        i = string.Pos(",");
        string.Delete(i,1);
        string.Insert(" ",i);
        }

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall RemoveChars(AnsiString string, AnsiString character)
{
    int i;
    while (string.Pos(character)) {
        i = string.Pos(character);
        string.Delete(i,1);
        }

    return string;
}
//---------------------------------------------------------------------------

AnsiString __fastcall LeftString(AnsiString string, int Length)
{
    //returns a string which is the leftmost 'Length' characters of string
    AnsiString data;

    if (Length > 0) {
        if (Length < string.Length()) {
            data = string.SubString(1,Length);
            } else {
            data = string;
            }
        }

    return data;

}
//---------------------------------------------------------------------------

AnsiString __fastcall RightString(AnsiString string, int Length)
{
    //returns a string which is the rightmost 'Length' characters of string
    AnsiString data;

    if (Length > 0) {
        if (Length < string.Length()) {
            data = string.SubString(string.Length() - Length + 1,Length);
            } else {
            data = string;
            }
        }

    return data;

}
//---------------------------------------------------------------------------

AnsiString __fastcall ChopOffFirstChars(AnsiString string, int Length)
{
    //returns string without the first 'Length' chars
    AnsiString data;

    if ((string.Length() - Length) > 0) {
        data = string.SubString(Length + 1,string.Length() - Length);
        }

    return data;

}
//---------------------------------------------------------------------------

AnsiString __fastcall ChopOffLastChars(AnsiString string, int Length)
{
    //returns string without the last 'Length' chars
    AnsiString data;

    if ((string.Length() - Length) > 0) {
        data = string.SubString(1,string.Length() - Length);
        }

    return data;

}
//---------------------------------------------------------------------------

int __fastcall AnyLettersPresent(AnsiString string)
{
    //points to first occurrence of a letter
    char Letters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    int position = 0;
    int pointer = 0;
    string = string.UpperCase();

    while (pointer < 26 && position == 0) {
        position = string.Pos(AnsiString(Letters[pointer]));
        pointer++;
        }

    return position;
}
//---------------------------------------------------------------------------

int __fastcall AnyofTheseCharsPresent(AnsiString List, AnsiString string)
{
    //points to first occurrence of a letter
    //char Letters[] = List.c_str();
    int position = 0;
    int pointer = 0;
    int max = List.Length();
    string = string.UpperCase();

    while (pointer < max && position == 0) {
        position = string.Pos(List.SubString(pointer,1));
        pointer++;
        }

    return position;
}
//---------------------------------------------------------------------------

AnsiString __fastcall ReturnSpaceIfNoLength(AnsiString string)
{
    if (string.Length()) {
        return string;
        } else {
        return " ";
        }
}
//---------------------------------------------------------------------------

AnsiString __fastcall ReplaceChars(AnsiString string, AnsiString original, AnsiString replacement)
{
    int i;
    while (string.Pos(original)) {
        i = string.Pos(original);
        string.Delete(i,original.Length());
        string.Insert(replacement,i);
        }

    return string;

}
//---------------------------------------------------------------------------


AnsiString __fastcall DecimalPlaces(int decimalplaces, int value)
{
    //surfac es the FloatToStrF function with an easy use decimal places wrapper
    AnsiString string = FloatToStrF((double)value, ffFixed, 15, decimalplaces);

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall DecimalPlaces(int decimalplaces, double value)
{

    AnsiString string = FloatToStrF(value, ffFixed, 15, decimalplaces);

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall DP(int decimalplaces, double value)
{

    AnsiString string = FloatToStrF(value, ffFixed, 15, decimalplaces);

    return string;

}
//---------------------------------------------------------------------------

AnsiString __fastcall RemoveLeftChars(int numberofchars, AnsiString string)
{
    //cuts off numberofchars from the left of string
    if (string.Length() > numberofchars) {
        string = string.SubString(numberofchars + 1, string.Length()  - numberofchars);
        } else {
        string = "";
        }

    return string;
}
//---------------------------------------------------------------------------


TDate __fastcall GetDate(AnsiString date)
{
    //This function returns the date in TDate format from a string of the
    //format "dd mm yyyy", "dd mmm yyyy", "dd mm", "dd mmm", "yyyy"
    //Defaults to today if all else fails!
    AnsiString Date = date.Trim();
    Date = Date.Trim();
    Date = ReplaceChars(Date," ","/");
    TDateTime TempDate;

    try {
        //as a first stab, assume it's the
        //normal dd mm yyyy format
        TempDate = StrToDate(Date);
        }

    catch (...) {
        //maybe it's dd mmm yyyy format
        Date = Date.UpperCase();
        if (Date.Pos("JAN")) {
            Date = ReplaceChars(Date,"JAN","01");
            } else {
            if (Date.Pos("FEB")) {
                Date = ReplaceChars(Date,"FEB","02");
                } else {
                if (Date.Pos("MAR")) {
                    Date = ReplaceChars(Date,"MAR","03");
                    } else {
                    if (Date.Pos("APR")) {
                        Date = ReplaceChars(Date,"APR","04");
                        } else {
                        if (Date.Pos("MAY")) {
                            Date = ReplaceChars(Date,"MAY","05");
                            } else {
                            if (Date.Pos("JUN")) {
                                Date = ReplaceChars(Date,"JUN","06");
                                } else {
                                if (Date.Pos("JUL")) {
                                    Date = ReplaceChars(Date,"JUL","07");
                                    } else {
                                    if (Date.Pos("AUG")) {
                                        Date = ReplaceChars(Date,"AUG","08");
                                        } else {
                                        if (Date.Pos("SEP")) {
                                            Date = ReplaceChars(Date,"SEP","09");
                                            } else {
                                            if (Date.Pos("OCT")) {
                                                Date = ReplaceChars(Date,"OCT","10");
                                                } else {
                                                if (Date.Pos("NOV")) {
                                                    Date = ReplaceChars(Date,"NOV","11");
                                                    } else {
                                                    if (Date.Pos("DEC")) Date = ReplaceChars(Date,"DEC","12");
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        switch (Date.Length()) {
            case 3:
            case 4:     {
                        if (Date.Pos("/") == 2 || Date.Pos("/") == 3) {
                            //got day + month. Put in current year
                            Date += "/" + (TempDate.CurrentDate()).FormatString("yyyy");
                            } else {
                            if (!Date.Pos("/")) {
                                //just got year, put in 1 jan
                                Date = "01/01/" + Date;
                                }
                            }
                        }
                        break;
            case 5:     {
                        switch (Date.Pos("/")) {
                            case 1:     {
                                        //got year only
                                        Date = "01/01" + Date;
                                        }
                                        break;
                            case 2:
                            case 3:     {
                                        //got day + month. Put in current year
                                        Date += "/" + (TempDate.CurrentDate()).FormatString("yyyy");
                                        }
                                        break;
                            default:    break;
                            }
                        }
                        break;
            default:    break;
            }

        try {
            TempDate = StrToDate(Date);
            }

        catch (...) {
            //Aw hell. Date done gone fut. Put in today's date.
            TempDate = TempDate.CurrentDate();
            }
        }

    return (TDate) TempDate;
}
//---------------------------------------------------------------------------



AnsiString __fastcall HourMinSec(double Angle)
{
    //returns an hour(or degree) minute, second string of the decimal angle.
    AnsiString Text = AnsiString(floor(Angle)) + "'";
    Angle = (Angle - floor(Angle))*60;
    Text += AnsiString(floor(Angle)) + "'";
    Angle = (Angle - floor(Angle))*60;
    Text += DecimalPlaces(2,Angle) + "\"";

    return Text;

}
//---------------------------------------------------------------------------


AnsiString __fastcall ExtractString(AnsiString string, int Start, int Length)
{
    //extracts a string of length Length starting at Start from 'string'
    //So if text is 'Hello there World' and want ExtractPortion(string, 7, 3)
    //you get 'the' returned
    AnsiString Text;

    if (Start > 1 && Start < string.Length()) Text = ChopOffFirstChars(string, Start - 1);
    if (Length < Text.Length()) Text = LeftString(Text, Length);

    return Text;
}
//---------------------------------------------------------------------------

AnsiString __fastcall IntToHex(int value)
{
    AnsiString text;

    int leftover;
    while (value > 0) {
        leftover = value % 16;
        value -= leftover;
        switch (leftover) {
            case 15:    {
                        text = "F" + text;
                        }
                        break;
            case 14:    {
                        text = "E" + text;
                        }
                        break;
            case 13:    {
                        text = "D" + text;
                        }
                        break;
            case 12:    {
                        text = "C" + text;
                        }
                        break;
            case 11:    {
                        text = "B" + text;
                        }
                        break;
            case 10:    {
                        text = "A" + text;
                        }
                        break;
            case 9:
            case 8:
            case 7:
            case 6:
            case 5:
            case 4:
            case 3:
            case 2:
            case 1:     {
                        text = AnsiString(leftover) + text;
                        }
                        break;
            default:    {
                        text = "0" + text;
                        }
                        break;
            }
        value = value/16;
        }

    if (text.Length() %2) text = "0" + text;
    text = "#" + text;
    return text;
}
//---------------------------------------------------------------------------



