//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Transparency.h"
#include "StringHelp.h"
#include "GraphicsHelp.h"
#include <math.h>
#include <clipbrd.hpp>
#include "RestoreStitches.h"
#include <shellapi.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTransparencyForm *TransparencyForm;
//---------------------------------------------------------------------------
__fastcall TTransparencyForm::TTransparencyForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormCreate(TObject *Sender)
{
    ChartImage = new Graphics::TBitmap();
    OriginalImage = new Graphics::TBitmap();
    DisplayImage = new Graphics::TBitmap();
    scale = 1.0;
    SquareSize = 1;
    TopLeftX = 0;
    TopLeftY = 0;
    offsetx = 0;
    offsety = 0;
	crTargetCursor = 10;
    Screen->Cursors[crTargetCursor] = LoadCursor((void *)HInstance, "TARGET");
    Image1->Cursor = (TCursor) crTargetCursor;
    crDropperCursor = 11;
    Screen->Cursors[crDropperCursor] = LoadCursor((void *)HInstance, "DROPPER");
    crDrawCursor = 12;
    Screen->Cursors[crDrawCursor] = LoadCursor((void *)HInstance, "DRAWCURSOR");
    active = true;
    Application->OnHint = DisplayHint;
    SelectedColour = 0;
    rubberbanding = false;
    wipedout = true;
    RubberBand.startX = 0;
    RubberBand.startY = 0;
    RubberBand.endX = 0;
    RubberBand.endY = 0;
    RubberBand.startchartX = 0.0;
    RubberBand.startchartY = 0.0;
    RubberBand.endchartX = 0.0;
    RubberBand.endchartY = 0.0;
    preparingtoaddstitches = false;
    undostitchlength = 0;
    addstitches = false;
    deletingstitches = false;
    colourpicking = false;
    pixelediting = false;
    imagechanged = false;
    stitcheschanged = false;
    changingcolour = false;
    //active = true;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormDestroy(TObject *Sender)
{
    ChartImage->FreeImage();
    delete ChartImage;
    OriginalImage->FreeImage();
    delete OriginalImage;
    DisplayImage->FreeImage();
    delete DisplayImage;
    ColourList.Length = 0;
    UsedColourList.Length = 0;
    BackStitchList.Length = 0;
    BackStitchRedo.Length = 0;
    Application->OnHint = NULL;
    PixelEditUndo.Length = 0;
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CreateDisplayImage(void)
{

    Screen->Cursor = crHourGlass;
    DisplayImage->Width = OriginalImage->Width;
    DisplayImage->Height = OriginalImage->Height;

    int pos = TrackBar1->Position;
    int revpos = TrackBar1->Max - TrackBar1->Position;
    int limit = TrackBar1->Max - TrackBar1->Min;
    offsetx = (int)UpDown1->Position;
    offsety = -1 * (int)UpDown2->Position;
    int locx, locy;
    rgb pix1,pix2,pix3;
    for (int i = 0; i < OriginalImage->Width; i++) {
        locx = (int)(((double)(i - offsetx))/scale);
        for (int j = 0 ; j < OriginalImage->Height; j++) {
            locy = (int)(((double)(j - offsety))/scale);
            if (locy > -1 && locy < ChartImage->Height && locx > -1 && locx < ChartImage->Width ) {
                pix1 = ExtractRGB(OriginalImage->Canvas->Pixels[i][j]);
                pix2 = ExtractRGB(ChartImage->Canvas->Pixels[locx][locy]);
                pix3.R = (pix1.R * revpos + pix2.R * pos)/limit;
                pix3.G = (pix1.G * revpos + pix2.G * pos)/limit;
                pix3.B = (pix1.B * revpos + pix2.B * pos)/limit;
                DisplayImage->Canvas->Pixels[i][j] = GetColourFromRGB(pix3);
                } else {
                DisplayImage->Canvas->Pixels[i][j] = OriginalImage->Canvas->Pixels[i][j];
                }
            }
        }
    ShowDisplayImage();
    Screen->Cursor = crDefault;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::UpDown3Click(TObject *Sender,
      TUDBtnType Button)
{
    //changes the image scale
    scale = 1.0 + 0.05 * (double)UpDown3->Position;
    Edit1->OnChange = NULL;
    Edit1->Text = AnsiString(scale);
    Edit1->OnChange = Edit1Change;
    CreateDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Edit1Change(TObject *Sender)
{
    scale = ToDoubleDef(Edit1->Text,1.0);
    if (scale == 0.0) {
        scale = 0.01;
        Edit1->OnChange = NULL;
        Edit1->Text = AnsiString(scale);
        Edit1->OnChange = Edit1Change;
        }
    CreateDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
    CreateDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::TrackBar1Change(TObject *Sender)
{
    CreateDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::ShowDisplayImage(void)
{
    if (active) {
        Screen->Cursor = crHourGlass;
        //this draws a copy of the bitmap held in memory on the image's canvas
        //int maxX = 0;
        //int maxY = 0;
        //first, clear picture
        Image1->Picture->Bitmap->Width = Image1->Width;
        Image1->Picture->Bitmap->Height = Image1->Height;
        TRect Rectangle;
        Image1->Canvas->Brush->Color = Color;
        Rectangle.left = 0;
        Rectangle.right = Image1->Picture->Bitmap->Width;
        Rectangle.top = 0;
        Rectangle.bottom = Image1->Picture->Bitmap->Height;
        Image1->Canvas->FillRect(Rectangle);
        wipedout = true;

        if (DisplayImage->Width > 0 && DisplayImage->Height > 0) {
            /*
            //slow version
            maxX = TopLeftX + (int) (Image1->Width / SquareSize) + 1;
            if (maxX > DisplayImage->Width) maxX = DisplayImage->Width;
            maxY = TopLeftY + (int) (Image1->Height / SquareSize) + 1;
            if (maxY > DisplayImage->Height) maxY = DisplayImage->Height;
            TColor PixelColour;
            Image1->Canvas->Brush->Style = bsSolid;
            Image1->Picture->Bitmap->Width = (maxX-TopLeftX+1)*SquareSize;
            Image1->Picture->Bitmap->Height = (maxY-TopLeftY+1)*SquareSize;
            for (int j = TopLeftY; j < maxY; j++) {
                for (int i = TopLeftX; i < maxX; i++) {
                    PixelColour = DisplayImage->Canvas->Pixels[i][j];
                    Image1->Canvas->Brush->Color = PixelColour;
                    Rectangle.left = i*SquareSize - TopLeftX * SquareSize;
                    Rectangle.right = (i+1)*SquareSize  - TopLeftX * SquareSize;
                    Rectangle.top = j*SquareSize - TopLeftY * SquareSize;
                    Rectangle.bottom = (j+1)*SquareSize - TopLeftY * SquareSize;
                    Image1->Canvas->FillRect(Rectangle);
                    }
                }
            */
            //fast version
            Rectangle.left = -TopLeftX * SquareSize;
            Rectangle.top = -TopLeftY * SquareSize;
            Rectangle.right = Rectangle.left + SquareSize * DisplayImage->Width;
            Rectangle.bottom = Rectangle.top + SquareSize * DisplayImage->Height;
            Image1->Canvas->StretchDraw(Rectangle,DisplayImage);
            DrawStitches();
            }

    }
    Screen->Cursor = crDefault;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::UpDown4Click(TObject *Sender,
      TUDBtnType Button)
{
    SquareSize = (int) UpDown4->Position;
    ShowDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::SpeedButton1Click(TObject *Sender)
{
    UpDown1->Position = UpDown1->Position + ((short)(10));
    CreateDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::SpeedButton2Click(TObject *Sender)
{
    UpDown1->Position = UpDown1->Position - (short)(10);
    CreateDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::ScrollBar1Change(TObject *Sender)
{
    int result = DisplayImage->Width - Image1->Width / SquareSize;
    if (result < 0) result = 0;
    TopLeftX = (int)((double) result * ((double) ScrollBar1->Position) / ((double) ScrollBar1->Max));
    ShowDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::ScrollBar2Change(TObject *Sender)
{
    int result = DisplayImage->Height - Image1->Height / SquareSize;
    if (result < 0) result = 0;
    TopLeftY = (int)((double) result * ((double) ScrollBar2->Position) / ((double) ScrollBar2->Max));
    ShowDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::SpeedButton3Click(TObject *Sender)
{
    UpDown2->Position = UpDown2->Position + (short)(10);
    CreateDisplayImage();

}
//---------------------------------------------------------------------------
void __fastcall TTransparencyForm::SpeedButton4Click(TObject *Sender)
{
    UpDown2->Position = UpDown2->Position - (short)(10);
    CreateDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Exit1Click(TObject *Sender)
{
    ModalResult = mrOk;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Cancel1Click(TObject *Sender)
{
    ModalResult = mrCancel;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormResize(TObject *Sender)
{
    ShowDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FillColourList(void)
{
    FullPallette->Items->Clear();
    for (int i = 0; i < ColourList.Length; i++) {
        FullPallette->Items->Add(ColourList[i].DMCColourName);
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FillUsedColourList(void)
{
    CurrentPallette->Items->Clear();
    for (int i = 0; i < UsedColourList.Length; i++) {
        CurrentPallette->Items->Add(UsedColourList[i].DMCColourName);
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CurrentPalletteDrawItem(
      TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
    // Draw coloured rectangle background & name of colour
    TRect Rectangle = Rect;
    TColor TempColour = CurrentPallette->Canvas->Brush->Color;
    CurrentPallette->Canvas->Brush->Style = bsSolid;
    CurrentPallette->Canvas->Brush->Color = UsedColourList[Index].Colour;
    CurrentPallette->Canvas->FillRect(Rectangle);
    CurrentPallette->Canvas->Brush->Color = TempColour;
    if (UsedColourList[Index].DrawWhite) {
        CurrentPallette->Canvas->Font->Color = clWhite;
        } else {
        CurrentPallette->Canvas->Font->Color = clBlack;
        }
    CurrentPallette->Canvas->Brush->Style = bsClear;
    //for some reason the next line doesn't work
    //CurrentPallette->Canvas->TextRect(Rectangle,1,1,CurrentPallette->Items->Strings[Index]);
    DrawText(CurrentPallette->Canvas->Handle,("  " + CurrentPallette->Items->Strings[Index]).c_str(),-1,(LPRECT) &Rectangle,DT_VCENTER | DT_SINGLELINE);
    CurrentPallette->Canvas->Brush->Style = bsSolid;
    
}
//---------------------------------------------------------------------------
void __fastcall TTransparencyForm::FullPalletteDrawItem(
      TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
    // Draw coloured rectangle background & name of colour
    TRect Rectangle = Rect;
    TColor TempColour = FullPallette->Canvas->Brush->Color;
    FullPallette->Canvas->Brush->Style = bsSolid;
    FullPallette->Canvas->Brush->Color = ColourList[Index].Colour;
    FullPallette->Canvas->FillRect(Rectangle);
    FullPallette->Canvas->Brush->Color = TempColour;
    if (ColourList[Index].DrawWhite) {
        FullPallette->Canvas->Font->Color = clWhite;
        } else {
        FullPallette->Canvas->Font->Color = clBlack;
        }
    FullPallette->Canvas->Brush->Style = bsClear;
    //for some reason the next line doesn't work
    //FullPallette->Canvas->TextRect(Rectangle,1,1,FullPallette->Items->Strings[Index]);
    DrawText(FullPallette->Canvas->Handle,("  " + FullPallette->Items->Strings[Index]).c_str(),-1,(LPRECT) &Rectangle,DT_VCENTER | DT_SINGLELINE);
    FullPallette->Canvas->Brush->Style = bsSolid;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
    mouseposition Location = FindMousePosition(X,Y);
    DisplayMousePosition(Location);
    LastMouseMove.X = X;
    LastMouseMove.Y = Y;

    if (rubberbanding) {
        TPenMode oldmode = Image1->Picture->Bitmap->Canvas->Pen->Mode;
        Image1->Picture->Bitmap->Canvas->Pen->Mode = pmNot;
        if (!wipedout) {
            Image1->Picture->Bitmap->Canvas->MoveTo(RubberBand.startX,RubberBand.startY);
            Image1->Picture->Bitmap->Canvas->LineTo(RubberBand.endX,RubberBand.endY);
            wipedout = true;
            }
        mouseposition temp = FindMousePosition(X,Y);
        if (!temp.outofrange) {
            //make sure end of rubber band is in right place
            temp.X = RoundToNearestQuarter(temp.X);
            temp.Y = RoundToNearestQuarter(temp.Y);
            RubberBand.endchartX = temp.X;
            RubberBand.endchartY = temp.Y;
            chartposition temp2 = FindChartPosition(temp);
            RubberBand.endX = temp2.X;
            RubberBand.endY = temp2.Y;
            //make sure start of rubber band is in up to date if the image zooms in etc
            temp.X = RubberBand.startchartX;
            temp.Y = RubberBand.startchartY;
            temp2 = FindChartPosition(temp);
            RubberBand.startX = temp2.X;
            RubberBand.startY = temp2.Y;
            Image1->Picture->Bitmap->Canvas->MoveTo(RubberBand.startX,RubberBand.startY);
            Image1->Picture->Bitmap->Canvas->LineTo(RubberBand.endX,RubberBand.endY);
            wipedout = false;
            }
        Image1->Picture->Bitmap->Canvas->Pen->Mode = oldmode;
        }
}
//---------------------------------------------------------------------------

mouseposition  __fastcall TTransparencyForm::FindMousePosition(int X, int Y)
{
    //This is an interesting problem. It takes the mouse position & converts
    //it into the location on the chart image.
    //The chart image may be scaled and displaced relative to the original image
    //as well as the whole lot being scrolled & scaled in the image box.

    //Where the mouse is out of the chart image area, Location.outofrange is set to true.
    //The mouse position snaps to the nearest 0.25 stitches on the chart image.

    mouseposition Location;
    Location.outofrange = false;
    Location.X = (double) X;
    Location.Y = (double) Y;

    //Transform to DisplayImage co-ords
    Location.X = ((double)TopLeftX) + Location.X/((double)SquareSize);
    Location.Y = ((double)TopLeftY) + Location.Y/((double)SquareSize);

    //Transform to ChartImage co-ords
    Location.X = (Location.X - ((double)offsetx))/scale ;
    Location.Y = (Location.Y - ((double)offsety))/scale ;

    //now round to the nearest 0.25 of a stitch
    Location.X = RoundToNearestQuarter(Location.X);
    Location.Y = RoundToNearestQuarter(Location.Y);

    //now check if within ChartImage limits.
    if (Location.X < 0.0
        || Location.X > (double)ChartImage->Width
        || Location.Y < 0.0
        || Location.Y > (double)ChartImage->Height) {
            Location.outofrange = true;
            }

    return Location;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::DisplayMousePosition(mouseposition Location)
{
    if (!Location.outofrange) {
        StatusBar1->Panels->Items[1]->Text = " X = " + DP(2,Location.X);
        StatusBar1->Panels->Items[2]->Text = " Y = " + DP(2,Location.Y);
        } else {
        StatusBar1->Panels->Items[1]->Text = " X = ---";
        StatusBar1->Panels->Items[2]->Text = " Y = ---";
        }
}
//---------------------------------------------------------------------------

chartposition __fastcall TTransparencyForm::FindChartPosition(mouseposition Location)
{
    //This method does the opposite of  FindMousePosition(int X, int Y) in that
    //it finds the current Image1 co-ords of location
    mouseposition Image1Point;
    chartposition Image2Point;
    Image2Point.X = 0;
    Image2Point.Y = 0;

    if (!Location.outofrange) {
        //Transform to DisplayImage co-ords
        Image1Point.X = Location.X * scale + (double)offsetx;
        Image1Point.Y = Location.Y * scale + (double)offsety;

        //transform to Image1 positions
        Image1Point.X = (Image1Point.X - (double)TopLeftX)*((double)SquareSize);
        Image1Point.Y = (Image1Point.Y - (double)TopLeftY)*((double)SquareSize);

        Image2Point.X = (int) Image1Point.X;
        Image2Point.Y = (int) Image1Point.Y;
        }

    return Image2Point;
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormDeactivate(TObject *Sender)
{
    //if the bitmap is drawn as the form is resized on destruction, an error
    //occurs. This stops it.
    active = false;    
    
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormActivate(TObject *Sender)
{
    active = true;
}
//---------------------------------------------------------------------------

double __fastcall TTransparencyForm::RoundToNearestQuarter(double value)
{
    double result;

    result = floor(value * 4.0 + 0.5)/4.0;

    return result;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::DisplayHint(TObject *Sender)
{
    StatusBar1->Panels->Items[3]->Text = " " + GetLongHint(Application->Hint);

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CopyDisplayedImage1Click(
      TObject *Sender)
{
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    Graphics::TBitmap *Bitmap = new Graphics::TBitmap();
    try {
        // generate a clipboard format, with data and palette
        // save the data to the clipboard using that format and
        // the generated data
        Bitmap->Width = Image1->Picture->Bitmap->Width;
        Bitmap->Height = Image1->Picture->Bitmap->Height;
        Bitmap->Canvas->Draw(0,0,Image1->Picture->Bitmap);
        Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
        Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
        }

    catch (...) {
        ShowMessage("Sorry, Failed to copy image to clipboard.");
        }

    delete Bitmap;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CopyFullImage1Click(TObject *Sender)
{
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    Graphics::TBitmap *Bitmap = new Graphics::TBitmap();
    try {
        // generate a clipboard format, with data and palette
        // save the data to the clipboard using that format and
        // the generated data
        Bitmap->Width = DisplayImage->Width;
        Bitmap->Height = DisplayImage->Height;
        Bitmap->Canvas->Draw(0,0,DisplayImage);
        Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
        Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
        }

    catch (...) {
        ShowMessage("Sorry, Failed to copy image to clipboard.");
        }

    delete Bitmap;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CopyChartImage1Click(TObject *Sender)
{
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    Graphics::TBitmap *Bitmap = new Graphics::TBitmap();
    try {
        // generate a clipboard format, with data and palette
        // save the data to the clipboard using that format and
        // the generated data
        Bitmap->Width = ChartImage->Width;
        Bitmap->Height = ChartImage->Height;
        Bitmap->Canvas->Draw(0,0,ChartImage);
        Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
        Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
        }

    catch (...) {
        ShowMessage("Sorry, Failed to copy image to clipboard.");
        }

    delete Bitmap;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CopyHighResImage1Click(TObject *Sender)
{
    unsigned int DataHandle = 0;
    HPALETTE APalette;
    unsigned short MyFormat = CF_BITMAP;
    Graphics::TBitmap *Bitmap = new Graphics::TBitmap();
    try {
        // generate a clipboard format, with data and palette
        // save the data to the clipboard using that format and
        // the generated data
        Bitmap->Width = OriginalImage->Width;
        Bitmap->Height = OriginalImage->Height;
        Bitmap->Canvas->Draw(0,0,OriginalImage);
        Bitmap->SaveToClipboardFormat(MyFormat,DataHandle,APalette);
        Clipboard()->SetAsHandle(CF_BITMAP,DataHandle);
        }

    catch (...) {
        ShowMessage("Sorry, Failed to copy image to clipboard.");
        }

    delete Bitmap;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::CurrentPalletteDblClick(TObject *Sender)
{
    //selects the colour
    int pointer = 0;
    bool found = false;
    while (!found && pointer < CurrentPallette->Items->Count) {
        if (CurrentPallette->Selected[pointer]) {
            pointer--;
            found = true;
            }
        pointer++;
        }
    if (found) {
        SelectedColour = pointer;
        CurrentPallette->TopIndex = pointer;
        PrintSelectedColour();
        }
    TColor PickedColour = UsedColourList[pointer].Colour;
    found = false;
    pointer = 0;
    while (!found && pointer < ColourList.Length) {
        if (PickedColour == ColourList[pointer].Colour) {
            pointer--;
            found = true;
            }
        pointer++;
        }
    if (found) {
        FullPallette->ItemIndex = pointer;
        FullPallette->TopIndex = pointer;
        }




}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::PrintSelectedColour(void)
{
    //updates the status bar with the selected colour
    StatusBar1->Panels->Items[0]->Text = " Colour = " + UsedColourList[SelectedColour].DMCColourName;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FullPalletteDblClick(TObject *Sender)
{
    //adds a new colour to the used colour list and selects the colour
    int pointer = 0;
    bool found = false;
    while (!found && pointer < FullPallette->Items->Count) {
        if (FullPallette->Selected[pointer]) {
            pointer--;
            found = true;
            }
        pointer++;
        }
    if (found) {
        //now to check if this one appears in the used list & get its pointer too;
        int pointer2 = 0;
        found = false;
        while (!found && pointer2 < UsedColourList.Length) {
            if (ColourList[pointer].DMCColourName == UsedColourList[pointer2].DMCColourName) {
                pointer2--;
                found = true;
                }
            pointer2++;
            }
        TColor PickedColour = ColourList[pointer].Colour;
        if (found) {
            //Select this colour
            SelectedColour = pointer2;
            } else {
            SelectedColour = UsedColourList.Length;
            UsedColourList.Length++;
            UsedColourList[UsedColourList.Length - 1].DMCColourName = ColourList[pointer].DMCColourName;
            UsedColourList[UsedColourList.Length - 1].Colour = ColourList[pointer].Colour;
            UsedColourList[UsedColourList.Length - 1].DrawWhite = ColourList[pointer].DrawWhite;
            UsedColourList[UsedColourList.Length - 1].StitchCount = ColourList[pointer].StitchCount;
            FillUsedColourList();
            }
        StatusBar1->Panels->Items[1]->Text = " Colour = " + UsedColourList[SelectedColour].DMCColourName;
        found = false;
        pointer = 0;
        while (!found && pointer < UsedColourList.Length) {
            if (PickedColour == UsedColourList[pointer].Colour) {
                pointer--;
                found = true;
                }
            pointer++;
            }
        if (found) {
            CurrentPallette->ItemIndex = pointer;
            CurrentPallette->TopIndex = pointer;
            FullPallette->TopIndex = FullPallette->ItemIndex;
            }

        }


}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::DrawStitches(void)
{
    //This method draws the backstitches on the image

    int linewidth = 1;
    Image1->Picture->Bitmap->Canvas->Pen->Width = linewidth;
    chartposition Startc,Finishc;
    mouseposition Startm,Finishm;
    Startm.outofrange = false;
    Finishm.outofrange = false;

    for (int i = 0; i < BackStitchList.Length; i++) {
        Startm.X = BackStitchList[i].X1;
        Startm.Y = BackStitchList[i].Y1;
        Finishm.X = BackStitchList[i].X2;
        Finishm.Y = BackStitchList[i].Y2;
        Startc = FindChartPosition(Startm);
        Finishc = FindChartPosition(Finishm);
        Image1->Picture->Bitmap->Canvas->Pen->Color = UsedColourList[BackStitchList[i].ColourNumber].Colour;
        Image1->Picture->Bitmap->Canvas->MoveTo(Startc.X,Startc.Y);
        Image1->Picture->Bitmap->Canvas->LineTo(Finishc.X,Finishc.Y);
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::SetSpinButtons(void)
{
    //This is effectively a 'FormActivate', but may have some old data lying about that needs to be cleared up
    //on entry to the form, this sets the offsetx & offsety controlling spin buttons
    UpDown1->Position = (short) offsetx;
    UpDown2->Position = (short)(offsety * -1);
    //now to set the scale textbox
    Edit1->OnChange = NULL;
    Edit1->Text = scale;
    Edit1->OnChange = Edit1Change;
    StatusBar1->Panels->Items[0]->Text = " Colour = " + CurrentPallette->Items->Strings[SelectedColour];

    imagechanged = false;
    stitcheschanged = false;

    //resetting undo's
    BackStitchRedo.Length = 0;
    PixelEditUndo.Length = 0;

    //setting up colour lists
    if (SelectedColour < UsedColourList.Length) {
        CurrentPallette->ItemIndex = CurrentPallette->Items->IndexOf(UsedColourList[SelectedColour].DMCColourName);
        CurrentPallette->TopIndex = CurrentPallette->ItemIndex;
        FullPallette->ItemIndex = FullPallette->Items->IndexOf(UsedColourList[SelectedColour].DMCColourName);
        FullPallette->TopIndex = FullPallette->ItemIndex;
        }


}
//---------------------------------------------------------------------------


void __fastcall TTransparencyForm::Image1Click(TObject *Sender)
{
    if (preparingtoaddstitches) {
        mouseposition temp = FindMousePosition(LastMouseMove.X,LastMouseMove.Y);
        if (!temp.outofrange) {
            temp.X = RoundToNearestQuarter(temp.X);
            temp.Y = RoundToNearestQuarter(temp.Y);
            RubberBand.startchartX = temp.X;
            RubberBand.startchartY = temp.Y;
            chartposition temp2 = FindChartPosition(temp);
            RubberBand.startX = temp2.X;
            RubberBand.startY = temp2.Y;
            wipedout = true;
            preparingtoaddstitches = false;
            rubberbanding = true;
            }
        } else {
        if (rubberbanding) {
            //this is now the end of the line and the stitch can be added
            mouseposition temp = FindMousePosition(LastMouseMove.X,LastMouseMove.Y);
            if (!temp.outofrange) {
                temp.X = RoundToNearestQuarter(temp.X);
                temp.Y = RoundToNearestQuarter(temp.Y);
                RubberBand.endchartX = temp.X;
                RubberBand.endchartY = temp.Y;
                chartposition temp2 = FindChartPosition(temp);
                RubberBand.endX = temp2.X;
                RubberBand.endY = temp2.Y;
                //OK, the start & end of the back stitch is now in RubberBand. Time to convert into segments to be stitched.
                //now to go between the two points & see the length;
                double stitchlength = sqrt((RubberBand.endchartX - RubberBand.startchartX)* (RubberBand.endchartX - RubberBand.startchartX) + (RubberBand.endchartY - RubberBand.startchartY)* (RubberBand.endchartY - RubberBand.startchartY)) + 0.5;
                int steps = (int) (stitchlength/2.0);
                if (steps > 1) {
                    //break down the stitch length
                    double tempsnapx;
                    double tempsnapy;
                    double intersnapx = RubberBand.startchartX;
                    double intersnapy = RubberBand.startchartY;
                    double dx = RubberBand.endchartX - RubberBand.startchartX;
                    double dy = RubberBand.endchartY - RubberBand.startchartY;
                    undostitchlength = 0 ;
                    for (int i = 0; i < steps; i++) {
                        //GetSnapFloat(((float)(i+1))*(lx)/((float)steps)+RubberBandStartX,((float)(i+1))*(ly)/((float)steps)+RubberBandStartY);
                        tempsnapx = RoundToNearestQuarter(RubberBand.startchartX + ((double)(i+1)) * dx / ((double) steps));
                        tempsnapy = RoundToNearestQuarter(RubberBand.startchartY + ((double)(i+1)) * dy / ((double) steps));
                        if (tempsnapx != intersnapx || tempsnapy != intersnapy) {
                            BackStitchList.Length++;
                            BackStitchList[BackStitchList.Length - 1].X1 = intersnapx;
                            BackStitchList[BackStitchList.Length - 1].Y1 = intersnapy;
                            BackStitchList[BackStitchList.Length - 1].X2 = tempsnapx;
                            BackStitchList[BackStitchList.Length - 1].Y2 = tempsnapy;
                            BackStitchList[BackStitchList.Length - 1].ColourNumber = SelectedColour;
                            BackStitchList[BackStitchList.Length - 1].Colour = UsedColourList[SelectedColour].Colour;
                            intersnapx = tempsnapx;
                            intersnapy = tempsnapy;
                            undostitchlength++;
                            stitcheschanged = true;
                            }
                        }
                    } else {
                    //should not get here unless the stitch is fairly short
                    BackStitchList.Length++;
                    BackStitchList[BackStitchList.Length - 1].X1 = RubberBand.startchartX;
                    BackStitchList[BackStitchList.Length - 1].Y1 = RubberBand.startchartY;
                    BackStitchList[BackStitchList.Length - 1].X2 = RubberBand.endchartX;
                    BackStitchList[BackStitchList.Length - 1].Y2 = RubberBand.endchartY;
                    BackStitchList[BackStitchList.Length - 1].ColourNumber = SelectedColour;
                    BackStitchList[BackStitchList.Length - 1].Colour = UsedColourList[SelectedColour].Colour;
                    undostitchlength = 1;
                    stitcheschanged = true;
                    }


                //now to show all the stitches and start rubberbanding from the new position.
                RubberBand.startchartX = RubberBand.endchartX;
                RubberBand.startchartY = RubberBand.endchartY;
                RubberBand.startX = RubberBand.endX;
                RubberBand.startY = RubberBand.endY;
                rubberbanding = true;
                ShowDisplayImage();
                }
            } else {
            if (deletingstitches || changingcolour) {
                Screen->Cursor = crHourGlass;
                /*
                note, this uses a rather inefficient process as it checks each stitch
                to see if it's possible, so many which could not possibly be the one are
                checked. Basically it's the triumph of brute force and ignorance over finess.
                */
                mouseposition temp = FindMousePosition(LastMouseMove.X,LastMouseMove.Y);
                float px = (float) RoundToNearestQuarter(temp.X);
                float py = (float) RoundToNearestQuarter(temp.Y);
                //float px = ((float) X)/ ((float)SquareSize) + (float) TopLeftX;
                //float py = ((float) Y)/ ((float)SquareSize) + (float) TopLeftY;
                float tol = 0.5 * 0.5;
                float u, l, result;
                bool found = false;
                float mindist = tol;
                int pointer;
                float XS, XE, YE, YS, T;
                for (int i = 0; i < BackStitchList.Length; i++) {
                    XS = BackStitchList[i].X1;
                    YS = BackStitchList[i].Y1;
                    XE = BackStitchList[i].X2;
                    YE = BackStitchList[i].Y2;
                    u = ((px - XS) * (YE - YS) - (py - YS) * (XE - XS)) * ((px - XS) * (YE - YS) - (py - YS) * (XE - XS));
                    l = (XE - XS) * (XE - XS) + (YE - YS) * (YE - YS);
                    T = ((XE - px) * (XE - XS) + (YE - py) * (YE - YS))/l;
                    result = (float) fabs ((double)(u/l));
                    if (result < tol && result < mindist && T > -0.25 && T < 1.25) {
                        found = true;
                        pointer = i;
                        mindist = result;
                        }
                    }
                if (found) {
                    if (deletingstitches) {
                        //OK, found the stitch to be deleted, now deal with it.
                        if (pointer < BackStitchList.Length) {
                            if (BackStitchList.Length - pointer < undostitchlength) {
                                undostitchlength--;
                                }
                            BackStitchRedo.Length++;
                            BackStitchRedo[BackStitchRedo.Length - 1] = BackStitchList[pointer];
                            for (int i = pointer; i < BackStitchList.Length - 1; i++) {
                                BackStitchList[i] = BackStitchList[i + 1];
                                }
                            BackStitchList.Length--;
                            //ShowDisplayImage();
                            }
                        }
                    if (changingcolour) {
                        if (pointer < BackStitchList.Length && pointer > -1) {
                            BackStitchList[pointer].ColourNumber = SelectedColour;
                            BackStitchList[pointer].Colour = UsedColourList[SelectedColour].Colour;
                            }
                        }
                    stitcheschanged = true;
                    ShowDisplayImage();
                    }
                Screen->Cursor = crDefault;
                } else {
                if (colourpicking) {
                    mouseposition temp = FindMousePosition(LastMouseMove.X,LastMouseMove.Y);
                    if (!temp.outofrange) {
                        TColor localcolour = ChartImage->Canvas->Pixels[(int)temp.X][(int)temp.Y];
                        bool found = false;
                        int counter = 0;
                        while (!found && counter < UsedColourList.Length) {
                            if (localcolour == UsedColourList[counter].Colour) {
                                found = true;
                                counter--;
                                }
                            counter++;
                            }
                        if (found) {
                            SelectedColour = counter;
                            PrintSelectedColour();
                            CurrentPallette->ItemIndex = counter;
                            CurrentPallette->TopIndex = counter;
                            }
                        found = false;
                        counter = 0;
                        while (!found && counter < ColourList.Length) {
                            if (localcolour == ColourList[counter].Colour) {
                                counter--;
                                found = true;
                                }
                            counter++;
                            }
                        if (found) {
                            FullPallette->ItemIndex = counter;
                            FullPallette->TopIndex = counter;
                            }
                        colourpicking = false;
                        Image1->Cursor = (TCursor) crTargetCursor;
                        }
                    } else {
                    if (pixelediting) {
                        //find colour at current mouse location
                        mouseposition temp = FindMousePosition(LastMouseMove.X,LastMouseMove.Y);
                        if (!temp.outofrange) {
                            TColor localcolour = ChartImage->Canvas->Pixels[(int)temp.X][(int)temp.Y];
                            PixelEditUndo.Length++;
                            //save undo
                            PixelEditUndo[PixelEditUndo.Length-1].X = (int)temp.X;
                            PixelEditUndo[PixelEditUndo.Length-1].Y = (int)temp.Y;
                            PixelEditUndo[PixelEditUndo.Length-1].Colour = localcolour;
                            //change to new colour
                            ChartImage->Canvas->Pixels[(int)temp.X][(int)temp.Y] = UsedColourList[SelectedColour].Colour ;
                            //finish off change
                            imagechanged = true;
                            CreateDisplayImage();
                            }
                        }  //end of 'if (pixelediting) {}
                    } //end of 'if (colourpicking) {} else {}
                } //end of 'if (deletingstitches) {} else {}
            }  //end of 'if (rubberbanding) {} else {}
        }  //end of 'if (preparingtoaddstitches) {} else {}
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FormKeyPress(TObject *Sender, char &Key)
{
    if (Key == 27) {
        pixelediting = false;
        colourpicking = false;
        if (Image1->Cursor != crTargetCursor) Image1->Cursor = (TCursor) crTargetCursor;
        if (rubberbanding || addstitches) {
            rubberbanding = false;
            preparingtoaddstitches = false;
            addstitches = false;
            AddStitchesButton->Caption = "Add Stitches";
            ShowDisplayImage();
            }
        }
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Edit3Click(TObject *Sender)
{
    AddStitches1->Checked = addstitches;
    DeleteStitches1->Checked = deletingstitches;
    UndoLastAddStitchOperation1->Enabled = (undostitchlength != 0);
    DeleteAllStitches1->Enabled = (bool) BackStitchList.Length;
    SelectDeletedStitchestoRedo1->Enabled = (bool) BackStitchRedo.Length;
    PixelEditor1->Checked = pixelediting;
    UndolastPixelEdits1->Enabled = (bool) PixelEditUndo.Length;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::UndoLastAddStitchOperation1Click(
      TObject *Sender)
{
    //deletes the last batch of stitches done. After adding stitches
    BackStitchList.Length = BackStitchList.Length - undostitchlength;
    undostitchlength = 0;
    ShowDisplayImage();

}
//---------------------------------------------------------------------------


void __fastcall TTransparencyForm::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if (Key == 27) {
        if (rubberbanding || addstitches) {
            rubberbanding = false;
            preparingtoaddstitches = false;
            addstitches = false;
            AddStitchesButton->Caption = "Add Stitches";
            DeleteStitchesButton->Caption = "Delete Stitches";
            deletingstitches = false;
            colourpicking = false;
            pixelediting = false;
            if (Image1->Cursor != crTargetCursor) Image1->Cursor = (TCursor) crTargetCursor;
            ShowDisplayImage();
            }
        }

}
//---------------------------------------------------------------------------


void __fastcall TTransparencyForm::AddStitchesButtonClick(TObject *Sender)
{
    addstitches = !addstitches;
    DeleteStitchesButton->Caption = "Delete Stitches";
    ChangeColour->Caption = "Change Colour";
    changingcolour = false;
    deletingstitches = false;
    colourpicking = false;
    pixelediting = false;

    if (!addstitches) {
        rubberbanding = false;
        preparingtoaddstitches = false;
        ShowDisplayImage();
        AddStitchesButton->Caption = "Add Stitches";
        } else {
        preparingtoaddstitches = true;
        AddStitchesButton->Caption = "Stop Stitches";
        if (Image1->Cursor != crTargetCursor) Image1->Cursor = (TCursor) crTargetCursor;
        if (rubberbanding) {
            rubberbanding = false;
            ShowDisplayImage();
            }
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::DeleteStitches1Click(TObject *Sender)
{
    deletingstitches = !deletingstitches;
    rubberbanding = false;
    preparingtoaddstitches = false;
    AddStitchesButton->Caption = "Add Stitches";
    ChangeColour->Caption = "Change Colour";
    changingcolour = false;
    colourpicking = false;
    pixelediting = false;
    if (Image1->Cursor != crTargetCursor) Image1->Cursor = (TCursor) crTargetCursor;
    ShowDisplayImage();
    if (deletingstitches) {
        undostitchlength = false;
        DeleteStitchesButton->Caption = "Stop Deletion";
        } else {
        DeleteStitchesButton->Caption = "Delete Stitches";
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::DeleteAllStitches1Click(TObject *Sender)
{
    int originallength = BackStitchRedo.Length;
    BackStitchRedo.Length += BackStitchList.Length;
    for (int i = 0; i < BackStitchList.Length; i++) {
        BackStitchRedo[originallength + i] = BackStitchList[i];
        }
    BackStitchList.Length = 0;
    colourpicking = false;
    pixelediting = false;
    if (Image1->Cursor != crTargetCursor) Image1->Cursor = (TCursor) crTargetCursor;
    ShowDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::SelectDeletedStitchestoRedo1Click(
      TObject *Sender)
{
    TRestoreStitchesForm *RestoreStitchesForm = new TRestoreStitchesForm(this);

    //fill the array
    RestoreStitchesForm->Stitches.Length = BackStitchRedo.Length;
    for (int i = 0; i < BackStitchRedo.Length; i++) {
        RestoreStitchesForm->Stitches[i].Stitch = BackStitchRedo[i];
        RestoreStitchesForm->Stitches[i].Colour = UsedColourList[BackStitchRedo[i].ColourNumber].Colour;
        RestoreStitchesForm->Stitches[i].DrawWhite = UsedColourList[BackStitchRedo[i].ColourNumber].DrawWhite;
        RestoreStitchesForm->Stitches[i].selected = false;
        }

    //fill the listbox
    RestoreStitchesForm->FillList();

    //show
    if (RestoreStitchesForm->ShowModal() == mrOk) {
        bool changed = false;
        //some backstitches may need to be restored
        for (int i = RestoreStitchesForm->ListBox1->Items->Count - 1; i > -1; i--) {
            //if (RestoreStitchesForm->ListBox1->Selected[i]) {
            if (RestoreStitchesForm->Stitches[i].selected) {
                changed = true;
                //copy this stitch across to the back stitch array & delete from the redo list
                BackStitchList.Length++;
                BackStitchList[BackStitchList.Length - 1] = BackStitchRedo[i];
                for (int j = i; j < BackStitchRedo.Length - 1; j++) {
                    BackStitchRedo[j] = BackStitchRedo[j + 1];
                    }
                BackStitchRedo.Length-- ;
                }
            }
        if (changed) ShowDisplayImage();
        }


    delete RestoreStitchesForm;
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::PickColour1Click(TObject *Sender)
{
    colourpicking = true;
    pixelediting = false;
    Image1->Cursor = (TCursor) crDropperCursor;
    deletingstitches = false;
    rubberbanding = false;
    preparingtoaddstitches = false;
    AddStitchesButton->Caption = "Add Stitches";
    DeleteStitchesButton->Caption = "Delete Stitches";
    ChangeColour->Caption = "Change Colour";
    changingcolour = false;
    ShowDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::PixelEditor1Click(TObject *Sender)
{
    pixelediting = !pixelediting;

    colourpicking = false;
    deletingstitches = false;
    rubberbanding = false;
    preparingtoaddstitches = false;
    AddStitchesButton->Caption = "Add Stitches";
    DeleteStitchesButton->Caption = "Delete Stitches";
    ChangeColour->Caption = "Change Colour";
    changingcolour = false;

    if (pixelediting) {
        Image1->Cursor = (TCursor) crDrawCursor;
        //clear out all previous edit undo's
        PixelEditUndo.Length = 0;
        } else {
        Image1->Cursor = (TCursor) crTargetCursor;
        }
    ShowDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::UndolastPixelEdits1Click(
      TObject *Sender)
{
    //undoes the pixel edits
    for (int i = PixelEditUndo.Length - 1; i > -1; i--) {
        ChartImage->Canvas->Pixels[PixelEditUndo[i].X][PixelEditUndo[i].Y] = PixelEditUndo[i].Colour ;
        }
    PixelEditUndo.Length = 0;
    CreateDisplayImage();
}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::ChangeColourClick(TObject *Sender)
{
    addstitches = false;
    DeleteStitchesButton->Caption = "Delete Stitches";
    deletingstitches = false;
    colourpicking = false;
    pixelediting = false;

    rubberbanding = false;
    preparingtoaddstitches = false;
    AddStitchesButton->Caption = "Add Stitches";

    changingcolour = !changingcolour;
    if (changingcolour) {
        ChangeColour->Caption = "Stop Change";
        } else {
        ChangeColour->Caption = "Change Colour";
        }
    ShowDisplayImage();

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Edit2Change(TObject *Sender)
{
    if (Edit2->Text.ToIntDef(1) < 1) {
        Edit2->Text = AnsiString(UpDown4->Position);
        } else {
        UpDown4->Position = (short) Edit2->Text.ToIntDef(1);
        SquareSize = (int) UpDown4->Position;
        ShowDisplayImage();
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::FixButtonClick(TObject *Sender)
{
    SpeedButton2->Enabled = !SpeedButton2->Enabled;
    UpDown1->Enabled = SpeedButton2->Enabled;
    SpeedButton1->Enabled = SpeedButton2->Enabled;
    SpeedButton3->Enabled = SpeedButton2->Enabled;
    UpDown2->Enabled = SpeedButton2->Enabled;
    SpeedButton4->Enabled = SpeedButton2->Enabled;
    UpDown1->Enabled = SpeedButton2->Enabled;
    Edit1->Enabled = SpeedButton2->Enabled;
    if (SpeedButton2->Enabled) {
        FixButton->Caption = "Disable";
        FixButton->Hint = "Disable Chart-High Res. image movement.";
        Edit1->Color = clWindow;
        } else {
        FixButton->Caption = "Enable";
        FixButton->Hint = "Enable Chart-High Res. image movement.";
        Edit1->Color = clBtnFace;
        }

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::Help2Click(TObject *Sender)
{
    AnsiString Address = ExtractFileDir(Application->ExeName) + "\\BackStitch.chm";
    AnsiString Location = "mk:@MSITStore:" + Address + "::/highres.html#start";
    ShellExecute(Application->Handle,"open",Location.c_str(),Location.c_str(),NULL,SW_SHOW);

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::PopupMenu2Popup(TObject *Sender)
{
    AnsiString text = StatusBar1->Panels->Items[3]->Text.Trim();
    HelpItem->Tag = -1;
    if (text == TrackBar1->Hint) HelpItem->Tag = 0;
    if (text == SpeedButton2->Hint) HelpItem->Tag = 1;
    if (text == Edit1->Hint) HelpItem->Tag = 2;
    if (text == FixButton->Hint) HelpItem->Tag = 3;
    if (text == Edit2->Hint) HelpItem->Tag = 4;
    if (text == AddStitchesButton->Hint || text == ChangeColour->Hint || text == BitBtn1->Hint || text == DeleteStitchesButton->Hint || text == BitBtn2->Hint) HelpItem->Tag = 5;

}
//---------------------------------------------------------------------------

void __fastcall TTransparencyForm::HelpItemClick(TObject *Sender)
{
    //Context sensitive help. Pain doing it this way, but can't do it for CHM files through the normal procedure.
    int Tag = dynamic_cast <TMenuItem *>(Sender)->Tag;
    AnsiString Address = ExtractFileDir(Application->ExeName) + "\\BackStitch.chm";
    AnsiString Location;
    switch (Tag) {
        case 0:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#transparency";
                    }
                    break;
        case 1:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#moving";
                    }
                    break;
        case 2:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#scalechart";
                    }
                    break;
        case 3:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#disable";
                    }
                    break;
        case 4:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#scaleimage";
                    }
                    break;
        case 5:     {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#action";
                    }
                    break;
        default:    {
                    Location = "mk:@MSITStore:" + Address + "::/highres.html#start";
                    }
                    break;
        }

    ShellExecute(Application->Handle,"open",Location.c_str(),Location.c_str(),NULL,SW_SHOW);

}
//---------------------------------------------------------------------------

